CREATE OR REPLACE PACKAGE GSIMONET.UTILITATS_PKG
AS
   /******************************************************************************
     NOM:       UTILITATS_PKG
     PROPSIT:  Package que cont tots els procediments genrics que es poden
                usar en qualsevol aplicaci i tamb cridats des de l'aplicaci
                superior
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Inserta logs a la taula de LOGS
    *  Parmetres d'entrada
    *  p_nompro : Nom del procediment
    *  p_log    : Variables d'entrada i sortida del procediments + els seus valors
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    * p_insert: 'S' si COMMIT
                'N' no COMMIT
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/

   PROCEDURE LOGS_INSERT (p_nompro   IN LOGS.NOM_PROCEDIMENT%TYPE,
                          p_log      IN LOGS.LOG%TYPE,
                          p_rsp      IN LOGS.RSP%TYPE,
                          p_insert   IN VARCHAR2 DEFAULT 'S' );

   /* ***************************************************************************
       *  Descripci: obt el missatge de l'error codificat
    *  Parmetres d'entrada
    *  p_codcod : Codi error
    *  Retorna Descripci de l'ERROR
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_MISSATGE_ERROR (p_cod IN ERROR.CODI%TYPE)
      RETURN ERROR.DESCRIPCIO%TYPE;

   /* ***************************************************************************
      *  Descripci: Comprova si existeix el patrocinador
   *  Parmetres d'entrada
   *  p_codi: Codi de patrocinador
   *  Retorna: 'S' si el troba, 'N' en cas contrari
   *
   *  Autor            Data         Descripci
   *------------    ------------    ------------------------
   * GSIMONET        01-04-2015   Creaci del procediment
   ****************************************************************************/
   FUNCTION EXISTEIX_PATROCINADOR (p_codi PATROCINADOR.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix la formacio
    *  Parmetres d'entrada
    *  p_codpil: Codi del pilot
    *  p_codequ: Codi de l'equip
    *  p_anyfor: Any de la formaci
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_FORMACIO (p_codpil   IN FORMACIO.CODI_PILOT%TYPE,
                               p_codequ   IN FORMACIO.CODI_EQUIP%TYPE,
                               p_anyfor   IN FORMACIO.ANY_FORMACIO%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix el cotxe per a l'equip
    *  Parmetres d'entrada
    *  p_codcot: Codi del cotxe
    *  p_codequ: Codi de l'equip
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_COTXE (p_codcot   IN COTXE.CODI%TYPE,
                            p_codequ   IN EQUIP.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix la carrera a la competici
    *  Parmetres d'entrada
    *  p_codcom: Codi de la competici
    *  p_codcar: Codi de la carrera
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_CARRERA (p_codcom   IN COMPETICIO.CODI%TYPE,
                              p_codcar   IN CARRERA.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix el component
    *  Parmetres d'entrada
    *  p_codcpt: Codi del component
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_COMPONENT (p_codcpt IN COMPONENT.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix la telemetria al resultat
    *  Parmetres d'entrada
    *  p_codres: Codi del resultat
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_TELEMETRIA_RES (
      p_codres   IN TELEMETRIA.CODI_RESULTAT%TYPE
   )
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir el fabricant del component
    *  Parmetres d'entrada
    *  p_codcpt: Codi del component
    *  Retorna: Fabricant si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_FABRICANT (p_codcpt IN COMPONENT.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir la fila del resultat
    *  Parmetres d'entrada
    *  p_codequ: Codi equip
    *  p_codpil: Codi pilot
    *  p_codcot: Codi cotxe
    *  p_codcom: Codi competici
    *  p_codcar: Codi carrera
    *  p_datcar: Data carrera
    *  Retorna: RESULTAT%ROWTYPE si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_RESULTAT (p_codequ   IN RESULTAT.CODI_EQUIP%TYPE,
                              p_codpil   IN RESULTAT.CODI_PILOT%TYPE,
                              p_codcot   IN RESULTAT.CODI_COTXE%TYPE,
                              p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
                              p_codcar   IN RESULTAT.CODI_CARRERA%TYPE,
                              p_datcar   IN RESULTAT.DATA_CARRERA%TYPE)
      RETURN RESULTAT%ROWTYPE;

   /* ***************************************************************************
       *  Descripci: Obtenir la fila de telemetria
    *  Parmetres d'entrada
    *  p_codtel: Codi telemetria
    *  Retorna: TELEMETRIA%ROWTYPE si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_TELEMETRIA (p_codtel IN TELEMETRIA.CODI%TYPE)
      RETURN TELEMETRIA%ROWTYPE;

   /* ***************************************************************************
       *  Descripci: Obtenir el codi de seqncia de EST_VOLTES_RAPIDES
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codcar: Codi de carrera
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_VOLTES_RAPIDES (
      p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN RESULTAT.CODI_CARRERA%TYPE
   )
      RETURN INTEGER;

   /* ***************************************************************************
       *  Descripci: Obtenir el codi de seqncia de EST_TEMP_MAXIMES
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codcar: Codi de carrera
    *  p_codcot: Codi del cotxe
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_TEMP_MAXIMES (
      p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN RESULTAT.CODI_CARRERA%TYPE,
      p_codcot   IN RESULTAT.CODI_COTXE%TYPE
   )
      RETURN INTEGER;

   /* ***************************************************************************
       *  Descripci: Obtenir el codi de seqncia de EST_COM_PIL
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codequ: Codi de l'equip
    *  p_codpil: Codi del pilot
    *  p_codcot: Codi del cotxe
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_COM_PIL (p_anycom   IN RESULTAT.ANY_FORMACIO%TYPE,
                                 p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
                                 p_codequ   IN RESULTAT.CODI_EQUIP%TYPE,
                                 p_codpil   IN RESULTAT.CODI_PILOT%TYPE,
                                 p_codcot   IN RESULTAT.CODI_COTXE%TYPE)
      RETURN INTEGER;

   /* ***************************************************************************
       *  Descripci: Obtenir el ncodi de la seqncia de EST_NUM_COMP_FABRICANT
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codfab: Codi del fabricant
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_NUM_COMP_FABRICANT (p_codcom    COMPETICIO.CODI%TYPE,
                                            p_codfab    FABRICANT.CODI%TYPE)
      RETURN INTEGER;

   /* ***************************************************************************
       *  Descripci: Obtenir el nom de l'equip
    *  Parmetres d'entrada
    *  p_codequ: Codi de l'equip.
    *  Retorna: Nom de l'equip
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_NOM_EQUIP (p_codequ IN EQUIP.CODI%TYPE)
      RETURN EQUIP.NOM%TYPE;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del cotxe
    *  Parmetres d'entrada
    *  p_codcot: Codi del cotxe.
    *  Retorna: Model + Data debut
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_COTXE (p_codcot IN COTXE.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del pilot
    *  Parmetres d'entrada
    *  p_codpil: Codi del PILOT
    *  Retorna: Nom + nacionalitat
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_PILOT (p_codpil IN PILOT.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del fabricant
    *  Parmetres d'entrada
    *  p_codfab: Codi del fabricant
    *  Retorna: Nom + NIF
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_FABRICANT (p_codfab IN FABRICANT.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades de la competici
    *  Parmetres d'entrada
    *  p_codcom: Codi de de la competici
    *  Retorna: Nom + Categoria
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_COMPETICIO (p_codcom IN COMPETICIO.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades de la carrera
    *  Parmetres d'entrada
    *  p_codcar: Codi de de la carrera
    *  Retorna: Nom + Data + Longitud
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_CARRERA (p_codcar IN CARRERA.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades de l'equip
    *  Parmetres d'entrada
    *  p_codequ: Codi de l'equip
    *  Retorna: Nom + Seu
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_EQUIP (p_codequ IN EQUIP.CODI%TYPE)
      RETURN VARCHAR2;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del patrocinador
    *  Parmetres d'entrada
    *  p_codpat: Codi del patrocinador
    *  Retorna: Nom + Sector
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_PATROCINADOR (p_codpat IN PATROCINADOR.CODI%TYPE)
      RETURN VARCHAR2;
END UTILITATS_PKG;
/

CREATE OR REPLACE PACKAGE BODY GSIMONET.UTILITATS_PKG
AS
   /******************************************************************************
     NOM:       UTILITATS_PKG
     PROPSIT:  Package que cont tots els procediments genrics que es poden
                usar en qualsevol aplicaci i tamb cridats des de l'aplicaci
                superior
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Inserta logs a la taula de LOGS
    *  Parmetres d'entrada
    *  p_nompro : Nom del procediment
    *  p_log    : Variables d'entrada i sortida del procediments + els seus valors
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    * p_insert: 'S' si COMMIT
                'N' no COMMIT
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   PROCEDURE LOGS_INSERT (p_nompro   IN LOGS.NOM_PROCEDIMENT%TYPE,
                          p_log      IN LOGS.LOG%TYPE,
                          p_rsp      IN LOGS.RSP%TYPE,
                          p_insert   IN VARCHAR2 DEFAULT 'S' )
   IS
      -- Declaraci de variables
      v_numerr   VARCHAR2 (30);
      v_deserr   ERROR.DESCRIPCIO%TYPE;
      v_rsp      LOGS.RSP%TYPE;
   BEGIN
      INSERT INTO LOGS (NOM_PROCEDIMENT, LOG, RSP)
        VALUES   (p_nompro, p_log, p_rsp);

      IF p_insert = 'S'
      THEN
         COMMIT;
      END IF;
   EXCEPTION
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_deserr := SUBSTR (SQLERRM, 1, 100);
         v_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ': ' || v_deserr;
         DBMS_OUTPUT.put_line (v_rsp);
   END LOGS_INSERT;

   /* ***************************************************************************
          *  Descripci: obt el missatge de l'error codificat
    *  Parmetres d'entrada
    *  p_codcod : Codi error
    *  Retorna Descripci de l'ERROR
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_MISSATGE_ERROR (p_cod IN ERROR.CODI%TYPE)
      RETURN ERROR.DESCRIPCIO%TYPE
   IS
      CURSOR c_err
      IS
         SELECT   DESCRIPCIO
           FROM   ERROR ERR
          WHERE   ERR.CODI = p_cod;

      v_err   c_err%ROWTYPE;
   BEGIN
      OPEN c_err;

      FETCH c_err INTO   v_err;

      IF c_err%FOUND
      THEN
         CLOSE c_err;

         RETURN v_err.DESCRIPCIO;
      END IF;

      CLOSE c_err;

      RETURN NULL;
   END OBTENIR_MISSATGE_ERROR;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix el patrocinador
    *  Parmetres d'entrada
    *  p_codi: Codi de patrocinador
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_PATROCINADOR (p_codi PATROCINADOR.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_pat
      IS
         SELECT   1
           FROM   PATROCINADOR PAT
          WHERE   PAT.CODI = p_codi;

      v_pat   c_pat%ROWTYPE;
   BEGIN
      OPEN c_pat;

      FETCH c_pat INTO   v_pat;

      IF c_pat%FOUND
      THEN
         CLOSE c_pat;

         RETURN 'S';
      ELSE
         CLOSE c_pat;

         RETURN 'N';
      END IF;
   END EXISTEIX_PATROCINADOR;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix la formacio
    *  Parmetres d'entrada
    *  p_codpil: Codi del pilot
    *  p_codequ: Codi de l'equip
    *  p_anyfor: Any de la formaci
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_FORMACIO (p_codpil   IN FORMACIO.CODI_PILOT%TYPE,
                               p_codequ   IN FORMACIO.CODI_EQUIP%TYPE,
                               p_anyfor   IN FORMACIO.ANY_FORMACIO%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_for
      IS
         SELECT   1
           FROM   FORMACIO FORMA
          WHERE       FORMA.CODI_PILOT = p_codpil
                  AND FORMA.CODI_EQUIP = p_codequ
                  AND FORMA.ANY_FORMACIO = p_anyfor;

      v_for   c_for%ROWTYPE;
   BEGIN
      OPEN c_for;

      FETCH c_for INTO   v_for;

      IF c_for%FOUND
      THEN
         CLOSE c_for;

         RETURN 'S';
      ELSE
         CLOSE c_for;

         RETURN 'N';
      END IF;
   END EXISTEIX_FORMACIO;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix el cotxe per a l'equip
    *  Parmetres d'entrada
    *  p_codcot: Codi del cotxe
    *  p_codequ: Codi de l'equip
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_COTXE (p_codcot   IN COTXE.CODI%TYPE,
                            p_codequ   IN EQUIP.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_for
      IS
         SELECT   1
           FROM   COTXE COT
          WHERE   COT.CODI = p_codcot AND COT.CODI_EQUIP = p_codequ;

      v_for   c_for%ROWTYPE;
   BEGIN
      OPEN c_for;

      FETCH c_for INTO   v_for;

      IF c_for%FOUND
      THEN
         CLOSE c_for;

         RETURN 'S';
      ELSE
         CLOSE c_for;

         RETURN 'N';
      END IF;
   END EXISTEIX_COTXE;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix la carrera a la competici
    *  Parmetres d'entrada
    *  p_codcom: Codi de la competici
    *  p_codcar: Codi de la carrera
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_CARRERA (p_codcom   IN COMPETICIO.CODI%TYPE,
                              p_codcar   IN CARRERA.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_for
      IS
         SELECT   1
           FROM   CARRERA CAR
          WHERE   CAR.CODI = p_codcar AND CAR.CODI_COMPETICIO = p_codcom;

      v_for   c_for%ROWTYPE;
   BEGIN
      OPEN c_for;

      FETCH c_for INTO   v_for;

      IF c_for%FOUND
      THEN
         CLOSE c_for;

         RETURN 'S';
      ELSE
         CLOSE c_for;

         RETURN 'N';
      END IF;
   END EXISTEIX_CARRERA;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix la telemetria al resultat
    *  Parmetres d'entrada
    *  p_codres: Codi del resultat
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_TELEMETRIA_RES (
      p_codres   IN TELEMETRIA.CODI_RESULTAT%TYPE
   )
      RETURN VARCHAR2
   IS
      CURSOR c_for
      IS
         SELECT   1
           FROM   TELEMETRIA TEL
          WHERE   TEL.CODI_RESULTAT = p_codres;

      v_for   c_for%ROWTYPE;
   BEGIN
      OPEN c_for;

      FETCH c_for INTO   v_for;

      IF c_for%FOUND
      THEN
         CLOSE c_for;

         RETURN 'S';
      ELSE
         CLOSE c_for;

         RETURN 'N';
      END IF;
   END EXISTEIX_TELEMETRIA_RES;

   /* ***************************************************************************
       *  Descripci: Comprova si existeix el component
    *  Parmetres d'entrada
    *  p_codcpt: Codi del component
    *  Retorna: 'S' si el troba, 'N' en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION EXISTEIX_COMPONENT (p_codcpt IN COMPONENT.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_for
      IS
         SELECT   1
           FROM   COMPONENT CPT
          WHERE   CPT.CODI = p_codcpt;

      v_for   c_for%ROWTYPE;
   BEGIN
      OPEN c_for;

      FETCH c_for INTO   v_for;

      IF c_for%FOUND
      THEN
         CLOSE c_for;

         RETURN 'S';
      ELSE
         CLOSE c_for;

         RETURN 'N';
      END IF;
   END EXISTEIX_COMPONENT;

   /* ***************************************************************************
       *  Descripci: Obtenir el fabricant del component
    *  Parmetres d'entrada
    *  p_codcpt: Codi del component
    *  Retorna: Fabricant si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_FABRICANT (p_codcpt IN COMPONENT.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_for
      IS
         SELECT   DISTINCT (CPT.CODI_FABRICANT)
           FROM   COMPONENT CPT
          WHERE   CPT.CODI = p_codcpt;

      v_for   c_for%ROWTYPE;
   BEGIN
      OPEN c_for;

      FETCH c_for INTO   v_for;

      IF c_for%FOUND
      THEN
         CLOSE c_for;

         RETURN v_for.CODI_FABRICANT;
      ELSE
         CLOSE c_for;

         RETURN NULL;
      END IF;
   END OBTENIR_FABRICANT;

   /* ***************************************************************************
       *  Descripci: Obtenir la fila del resultat
    *  Parmetres d'entrada
    *  p_codequ: Codi equip
    *  p_codpil: Codi pilot
    *  p_codcot: Codi cotxe
    *  p_codcom: Codi competici
    *  p_codcar: Codi carrera
    *  p_datcar: Data carrera
    *  Retorna: RESULTAT%ROWTYPE si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_RESULTAT (p_codequ   IN RESULTAT.CODI_EQUIP%TYPE,
                              p_codpil   IN RESULTAT.CODI_PILOT%TYPE,
                              p_codcot   IN RESULTAT.CODI_COTXE%TYPE,
                              p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
                              p_codcar   IN RESULTAT.CODI_CARRERA%TYPE,
                              p_datcar   IN RESULTAT.DATA_CARRERA%TYPE)
      RETURN RESULTAT%ROWTYPE
   IS
      CURSOR c_res
      IS
         SELECT   *
           FROM   RESULTAT RES
          WHERE       RES.CODI_EQUIP = p_codequ
                  AND RES.CODI_PILOT = p_codpil
                  AND RES.CODI_COTXE = p_codcot
                  AND RES.CODI_COMPETICIO = p_codcom
                  AND RES.CODI_CARRERA = p_codcar
                  AND RES.DATA_CARRERA = p_datcar;

      v_res   c_res%ROWTYPE;
      v_ret   RESULTAT%ROWTYPE;
   BEGIN
      v_ret := NULL;

      OPEN c_res;

      FETCH c_res INTO   v_res;

      IF c_res%FOUND
      THEN
         v_ret := v_res;
      END IF;

      CLOSE c_res;

      RETURN v_ret;
   END OBTENIR_RESULTAT;

   /* ***************************************************************************
       *  Descripci: Obtenir la fila de telemetria
    *  Parmetres d'entrada
    *  p_codtel: Codi telemetria
    *  Retorna: TELEMETRIA%ROWTYPE si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_TELEMETRIA (p_codtel IN TELEMETRIA.CODI%TYPE)
      RETURN TELEMETRIA%ROWTYPE
   IS
      CURSOR c_tel
      IS
         SELECT   *
           FROM   TELEMETRIA TEL
          WHERE   TEL.CODI = p_codtel;

      v_tel   c_tel%ROWTYPE;
      v_ret   TELEMETRIA%ROWTYPE;
   BEGIN
      v_ret := NULL;

      OPEN c_tel;

      FETCH c_tel INTO   v_tel;

      IF c_tel%FOUND
      THEN
         v_ret := v_tel;
      END IF;

      CLOSE c_tel;

      RETURN v_ret;
   END OBTENIR_TELEMETRIA;

   /* ***************************************************************************
       *  Descripci: Obtenir el codi de seqncia de EST_VOLTES_RAPIDES
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codcar: Codi de carrera
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_VOLTES_RAPIDES (
      p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN RESULTAT.CODI_CARRERA%TYPE
   )
      RETURN INTEGER
   IS
      CURSOR c_est
      IS
         SELECT   EST.CODI
           FROM   EST_VOLTES_RAPIDES EST
          WHERE   EST.CODI_COMPETICIO = p_codcom
                  AND EST.CODI_CARRERA = p_codcar;

      v_est   c_est%ROWTYPE;
      v_ret   INTEGER;
   BEGIN
      v_ret := NULL;

      OPEN c_est;

      FETCH c_est INTO   v_est;

      IF c_est%FOUND
      THEN
         v_ret := v_est.CODI;
      END IF;

      CLOSE c_est;

      RETURN v_ret;
   END OBTENIR_EST_VOLTES_RAPIDES;

   /* ***************************************************************************
       *  Descripci: Obtenir el codi de seqncia de EST_TEMP_MAXIMES
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codcar: Codi de carrera
    *  p_codcot: Codi del cotxe
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_TEMP_MAXIMES (
      p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN RESULTAT.CODI_CARRERA%TYPE,
      p_codcot   IN RESULTAT.CODI_COTXE%TYPE
   )
      RETURN INTEGER
   IS
      CURSOR c_est
      IS
         SELECT   EST.CODI
           FROM   EST_TEMP_MAXIMES EST
          WHERE       EST.CODI_COMPETICIO = p_codcom
                  AND EST.CODI_CARRERA = p_codcar
                  AND EST.CODI_COTXE = p_codcot;

      v_est   c_est%ROWTYPE;
      v_ret   INTEGER;
   BEGIN
      v_ret := NULL;

      OPEN c_est;

      FETCH c_est INTO   v_est;

      IF c_est%FOUND
      THEN
         v_ret := v_est.CODI;
      END IF;

      CLOSE c_est;

      RETURN v_ret;
   END OBTENIR_EST_TEMP_MAXIMES;

   /* ***************************************************************************
       *  Descripci: Obtenir el codi de seqncia de EST_COM_PIL
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codequ: Codi de l'equip
    *  p_codpil: Codi del pilot
    *  p_codcot: Codi del cotxe
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_COM_PIL (p_anycom   IN RESULTAT.ANY_FORMACIO%TYPE,
                                 p_codcom   IN RESULTAT.CODI_COMPETICIO%TYPE,
                                 p_codequ   IN RESULTAT.CODI_EQUIP%TYPE,
                                 p_codpil   IN RESULTAT.CODI_PILOT%TYPE,
                                 p_codcot   IN RESULTAT.CODI_COTXE%TYPE)
      RETURN INTEGER
   IS
      CURSOR c_est
      IS
         SELECT   EST.CODI
           FROM   EST_COM_PIL EST
          WHERE       EST.ANY_COMPETICIO = p_anycom
                  AND EST.CODI_COMPETICIO = p_codcom
                  AND EST.CODI_EQUIP = p_codequ
                  AND EST.CODI_PILOT = p_codpil
                  AND EST.CODI_COTXE = p_codcot;

      v_est   c_est%ROWTYPE;
      v_ret   INTEGER;
   BEGIN
      v_ret := NULL;

      OPEN c_est;

      FETCH c_est INTO   v_est;

      IF c_est%FOUND
      THEN
         v_ret := v_est.CODI;
      END IF;

      CLOSE c_est;

      RETURN v_ret;
   END OBTENIR_EST_COM_PIL;

   /* ***************************************************************************
       *  Descripci: Obtenir el ncodi de la seqncia de EST_NUM_COMP_FABRICANT
    *  Parmetres d'entrada
    *  p_codcom: Codi de competici
    *  p_codfab: Codi del fabricant
    *  Retorna: Codi si existeix, NULL en cas contrari
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_EST_NUM_COMP_FABRICANT (p_codcom    COMPETICIO.CODI%TYPE,
                                            p_codfab    FABRICANT.CODI%TYPE)
      RETURN INTEGER
   IS
      CURSOR c_est
      IS
         SELECT   EST.CODI
           FROM   EST_NUM_COMP_FABRICANT EST
          WHERE   EST.CODI_COMPETICIO = p_codcom
                  AND EST.CODI_FABRICANT = p_codfab;

      v_est   c_est%ROWTYPE;
      v_ret   INTEGER;
   BEGIN
      v_ret := NULL;

      OPEN c_est;

      FETCH c_est INTO   v_est;

      IF c_est%FOUND
      THEN
         v_ret := v_est.CODI;
      END IF;

      CLOSE c_est;

      RETURN v_ret;
   END OBTENIR_EST_NUM_COMP_FABRICANT;

   /* ***************************************************************************
       *  Descripci: Obtenir el nom de l'equip
    *  Parmetres d'entrada
    *  p_codequ: Codi de l'equip.
    *  Retorna: Nom de l'equip
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_NOM_EQUIP (p_codequ IN EQUIP.CODI%TYPE)
      RETURN EQUIP.NOM%TYPE
   IS
      CURSOR c_dad
      IS
         SELECT   EQU.NOM
           FROM   EQUIP EQU
          WHERE   EQU.CODI = p_codequ;

      v_dad   c_dad%ROWTYPE;
      v_ret   EQUIP.NOM%TYPE;
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.NOM;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_NOM_EQUIP;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del cotxe
    *  Parmetres d'entrada
    *  p_codcot: Codi del cotxe.
    *  Retorna: Model + Data debut
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_COTXE (p_codcot IN COTXE.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_dad
      IS
         SELECT      ' Model: '
                  || COT.MODEL
                  || ' Data debut: '
                  || COT.DATA_DEBUT
                     AS DADES_COTXE
           FROM   COTXE COT
          WHERE   COT.CODI = p_codcot;

      v_dad   c_dad%ROWTYPE;
      v_ret   VARCHAR2 (256);
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.DADES_COTXE;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_DADES_COTXE;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del pilot
    *  Parmetres d'entrada
    *  p_codpil: Codi del PILOT
    *  Retorna: Nom + nacionalitat
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_PILOT (p_codpil IN PILOT.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_dad
      IS
         SELECT      ' Nom: '
                  || PIL.NOM
                  || ' Nacionalitat: '
                  || PIL.NACIONALITAT
                     AS DADES_PILOT
           FROM   PILOT PIL
          WHERE   PIL.CODI = p_codpil;

      v_dad   c_dad%ROWTYPE;
      v_ret   VARCHAR2 (256);
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.DADES_PILOT;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_DADES_PILOT;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del fabricant
    *  Parmetres d'entrada
    *  p_codfab: Codi del fabricant
    *  Retorna: Nom + NIF
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_FABRICANT (p_codfab IN FABRICANT.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_dad
      IS
         SELECT   ' Nom: ' || FAB.NOM || ' NIF: ' || FAB.NIF
                     AS DADES_FABRICANT
           FROM   FABRICANT FAB
          WHERE   FAB.CODI = p_codfab;

      v_dad   c_dad%ROWTYPE;
      v_ret   VARCHAR2 (256);
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.DADES_FABRICANT;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_DADES_FABRICANT;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades de la competici
    *  Parmetres d'entrada
    *  p_codcom: Codi de de la competici
    *  Retorna: Nom + Categoria
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_COMPETICIO (p_codcom IN COMPETICIO.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_dad
      IS
         SELECT      ' Nom: '
                  || COM.DESCRIPCIO
                  || ' Categoria: '
                  || COM.CODI_CATEGORIA
                     AS DADES_COMPETICIO
           FROM   COMPETICIO COM
          WHERE   COM.CODI = p_codcom;

      v_dad   c_dad%ROWTYPE;
      v_ret   VARCHAR2 (256);
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.DADES_COMPETICIO;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_DADES_COMPETICIO;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades de la carrera
    *  Parmetres d'entrada
    *  p_codcar: Codi de de la carrera
    *  Retorna: Nom + Data + Longitud
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_CARRERA (p_codcar IN CARRERA.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_dad
      IS
         SELECT      ' Nom: '
                  || CAR.NOM
                  || ' Data '
                  || CAR.DATA_CARRERA
                  || ' Longitud: '
                  || CAR.LONGITUD_CIRCUIT
                     AS DADES_CARRERA
           FROM   CARRERA CAR
          WHERE   CAR.CODI = p_codcar;

      v_dad   c_dad%ROWTYPE;
      v_ret   VARCHAR2 (256);
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.DADES_CARRERA;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_DADES_CARRERA;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades de l'equip
    *  Parmetres d'entrada
    *  p_codequ: Codi de l'equip
    *  Retorna: Nom + Seu
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_EQUIP (p_codequ IN EQUIP.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_dad
      IS
         SELECT   ' Nom: ' || EQU.NOM || ' Seu: ' || EQU.SEU AS DADES_EQUIP
           FROM   EQUIP EQU
          WHERE   EQU.CODI = p_codequ;

      v_dad   c_dad%ROWTYPE;
      v_ret   VARCHAR2 (256);
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.DADES_EQUIP;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_DADES_EQUIP;

   /* ***************************************************************************
       *  Descripci: Obtenir les dades del patrocinador
    *  Parmetres d'entrada
    *  p_codpat: Codi del patrocinador
    *  Retorna: Nom + Sector
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci del procediment
    ****************************************************************************/
   FUNCTION OBTENIR_DADES_PATROCINADOR (p_codpat IN PATROCINADOR.CODI%TYPE)
      RETURN VARCHAR2
   IS
      CURSOR c_dad
      IS
         SELECT   ' Nom: ' || PAT.NOM || ' Sector: ' || PAT.SECTOR
                     AS DADES_PATROCINADOR
           FROM   PATROCINADOR PAT
          WHERE   PAT.CODI = p_codpat;

      v_dad   c_dad%ROWTYPE;
      v_ret   VARCHAR2 (256);
   BEGIN
      v_ret := NULL;

      OPEN c_dad;

      FETCH c_dad INTO   v_dad;

      IF c_dad%FOUND
      THEN
         v_ret := v_dad.DADES_PATROCINADOR;
      END IF;

      CLOSE c_dad;

      RETURN v_ret;
   END OBTENIR_DADES_PATROCINADOR;
END UTILITATS_PKG;
/

CREATE OR REPLACE PACKAGE GSIMONET.EQUIP_PKG
AS
   /******************************************************************************
     NOM:       EQUIP_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que l'actor
                Administrador FIA i Equip poden executar des de l'aplicaci superior.
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Afegeix un pilot a un equip
    *  Parmetros d'entrada
    *  p_codpil : Codi del pilot
    *  p_codequ : Codi de l'equip
    *  p_anyfor : Any en que es forma l'equip
    *  Parmetros de sortida
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   PROCEDURE AFEGIR_PILOT_EQUIP_COMPETICIO (
      p_codpil   IN     FORMACIO.CODI_PILOT%TYPE,
      p_codequ   IN     FORMACIO.CODI_EQUIP%TYPE,
      p_anyfor   IN     FORMACIO.ANY_FORMACIO%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   );
END EQUIP_PKG;
/

CREATE OR REPLACE PACKAGE BODY GSIMONET.EQUIP_PKG
AS
   /******************************************************************************
     NOM:       EQUIP_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que l'actor
                Administrador FIA i Equip poden executar des de l'aplicaci superior.
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Afegeix un pilot a un equip
    *  Parmetros d'entrada
    *  p_codpil : Codi del pilot
    *  p_codequ : Codi de l'equip
    *  p_anyfor : Any en que es forma l'equip
    *  Parmetros de sortida
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/

   PROCEDURE AFEGIR_PILOT_EQUIP_COMPETICIO (
      p_codpil   IN     FORMACIO.CODI_PILOT%TYPE,
      p_codequ   IN     FORMACIO.CODI_EQUIP%TYPE,
      p_anyfor   IN     FORMACIO.ANY_FORMACIO%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   )
   IS
      -- Cursor per comprovar quants de pilots t un equip per any (mxim 2)
      CURSOR c_for
      IS
         SELECT   COUNT ( * ) AS NUM_PILOTS
           FROM   FORMACIO FORMA
          WHERE   FORMA.CODI_EQUIP = p_codequ
                  AND FORMA.ANY_FORMACIO = p_anyfor;

      v_for      c_for%ROWTYPE;

      -- Cursor que comprova que el pilot no est fitxat per un altre equip
      -- dintre de la temporada passada per parmetre.
      CURSOR c_pil
      IS
         SELECT   1
           FROM   FORMACIO FORMA
          WHERE   FORMA.CODI_PILOT = p_codpil
                  AND FORMA.ANY_FORMACIO = p_anyfor;

      v_pil      c_pil%ROWTYPE;
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_formacio_completa EXCEPTION;
      e_pilot_te_equip_temporada EXCEPTION;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'AFEGIR_PILOT_EQUIP_COMPETICIO';
      v_log :=
            'p_codpil  ->'
         || p_codpil
         || ', p_codequ ->'
         || p_codequ
         || ', p_anyfor ->'
         || p_anyfor;


      OPEN c_for;

      FETCH c_for INTO   v_for;

      CLOSE c_for;

      OPEN c_pil;

      FETCH c_pil INTO   v_pil;

      -- Si el pilot ja t equip la durant la temporada passada per parmetre.
      IF c_pil%FOUND
      THEN
         CLOSE c_pil;

         RAISE e_pilot_te_equip_temporada;
      ELSE
         -- Si el pilot no t equip la durant la temporada passada per parmetre.
         CLOSE c_pil;

         -- Si el cursor retorna menys de dos pilots es fa l'insert.
         IF v_for.NUM_PILOTS < 2
         THEN
            INSERT INTO FORMACIO (CODI_EQUIP, CODI_PILOT, ANY_FORMACIO)
              VALUES   (p_codequ, p_codpil, p_anyfor);

            COMMIT;
            p_rsp := 'OK';
            UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
         ELSE
            RAISE e_formacio_completa;
         END IF;
      END IF;
   EXCEPTION
      WHEN e_formacio_completa
      THEN
         v_numerr := 'E001';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_pilot_te_equip_temporada
      THEN
         v_numerr := 'E002';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END AFEGIR_PILOT_EQUIP_COMPETICIO;
END EQUIP_PKG;
/

CREATE OR REPLACE PACKAGE GSIMONET.ESTADISTICA_PKG
AS
   /******************************************************************************
     NOM:       ESTADISTICA_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que serveixen
                per a actualitzar el mdul estadstic i que sn llanats pels
                triggers d'aquest.
   *******************************************************************************/

   FUNCTION ACT_NUM_CAR_GUANYADES (p_codcom   IN COMPETICIO.CODI%TYPE,
                                   p_codequ   IN EQUIP.CODI%TYPE,
                                   p_codpil   IN PILOT.CODI%TYPE,
                                   p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER;

   FUNCTION ACT_NUM_CAR_ACABADES (p_codcom   IN COMPETICIO.CODI%TYPE,
                                  p_codequ   IN EQUIP.CODI%TYPE,
                                  p_codpil   IN PILOT.CODI%TYPE,
                                  p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER;

   FUNCTION ACT_CARBURANT_GASTAT (p_codcom   IN COMPETICIO.CODI%TYPE,
                                  p_codequ   IN EQUIP.CODI%TYPE,
                                  p_codpil   IN PILOT.CODI%TYPE,
                                  p_codcot   IN COTXE.CODI%TYPE)
      RETURN FLOAT;

   FUNCTION ACT_VOLUM_TELEMETRIA (p_codcom   IN COMPETICIO.CODI%TYPE,
                                  p_codequ   IN EQUIP.CODI%TYPE,
                                  p_codpil   IN PILOT.CODI%TYPE,
                                  p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER;

   PROCEDURE ACT_DINERS_PATROCINADOR (
      p_codcom   IN     COMPETICIO.CODI%TYPE,
      p_codequ   IN     EQUIP.CODI%TYPE,
      p_codpil   IN     PILOT.CODI%TYPE,
      p_codcot   IN     COTXE.CODI%TYPE,
      p_valpat      OUT PATROCINADOR.VALOR_PATROCINI%TYPE,
      p_codpat      OUT PATROCINADOR.CODI%TYPE
   );

   FUNCTION ACT_TEMPERATURA_MAX (p_codcom   IN COMPETICIO.CODI%TYPE,
                                 p_codequ   IN EQUIP.CODI%TYPE,
                                 p_codpil   IN PILOT.CODI%TYPE,
                                 p_codcot   IN COTXE.CODI%TYPE)
      RETURN FLOAT;

   FUNCTION ACT_PER_COM_DEFECTUOS (p_codcom   IN COMPETICIO.CODI%TYPE,
                                   p_codequ   IN EQUIP.CODI%TYPE,
                                   p_codpil   IN PILOT.CODI%TYPE,
                                   p_codcot   IN COTXE.CODI%TYPE)
      RETURN FLOAT;

   FUNCTION ACT_CLASSIFICACIO (p_codcom   IN COMPETICIO.CODI%TYPE,
                               p_codequ   IN EQUIP.CODI%TYPE,
                               p_codpil   IN PILOT.CODI%TYPE,
                               p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER;

   PROCEDURE ACT_VOLTA_RAPIDA (p_codcom   IN     COMPETICIO.CODI%TYPE,
                               p_codcar   IN     CARRERA.CODI%TYPE,
                               p_volrap      OUT RESULTAT.VOLTA_RAPIDA%TYPE,
                               p_datcar      OUT CARRERA.DATA_CARRERA%TYPE,
                               p_codpil      OUT PILOT.CODI%TYPE,
                               p_codequ      OUT PILOT.CODI%TYPE);

   PROCEDURE ACT_TEMP_MAXIMA (p_codcom   IN     COMPETICIO.CODI%TYPE,
                              p_codcar   IN     CARRERA.CODI%TYPE,
                              p_codcot   IN     COTXE.CODI%TYPE,
                              p_datcar      OUT CARRERA.DATA_CARRERA%TYPE,
                              p_codpil      OUT PILOT.CODI%TYPE,
                              p_temmax      OUT TELEMETRIA.VALOR%TYPE);

   FUNCTION ACT_NUM_COMPONENTS (p_codcom   IN COMPETICIO.CODI%TYPE,
                                p_codfab   IN FABRICANT.CODI%TYPE)
      RETURN INTEGER;

   PROCEDURE ACT_EST_VOLTES_RAPIDES (
      p_est   IN     EST_VOLTES_RAPIDES%ROWTYPE,
      p_rsp      OUT LOGS.RSP%TYPE
   );

   PROCEDURE ACT_EST_TEMP_MAXIMES (p_est   IN     EST_TEMP_MAXIMES%ROWTYPE,
                                   p_rsp      OUT LOGS.RSP%TYPE);

   PROCEDURE ACT_EST_COM_PIL (p_est   IN     EST_COM_PIL%ROWTYPE,
                              p_rsp      OUT LOGS.RSP%TYPE);

   PROCEDURE ACT_EST_NUM_COMP_FABRICANT (
      p_est   IN     EST_NUM_COMP_FABRICANT%ROWTYPE,
      p_rsp      OUT LOGS.RSP%TYPE
   );
END ESTADISTICA_PKG;
/

CREATE OR REPLACE PACKAGE BODY GSIMONET.ESTADISTICA_PKG
AS
   /******************************************************************************
     NOM:       ESTADISTICA_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que serveixen
                per a actualitzar el mdul estadstic i que sn llanats pels
                triggers d'aquest.
   *******************************************************************************/

   FUNCTION ACT_NUM_CAR_GUANYADES (p_codcom   IN COMPETICIO.CODI%TYPE,
                                   p_codequ   IN EQUIP.CODI%TYPE,
                                   p_codpil   IN PILOT.CODI%TYPE,
                                   p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER
   IS
      CURSOR c_cars
      IS
         SELECT   COUNT ( * ) NUM_CAR_GUANYADES
           FROM   RESULTAT RES
          WHERE       RES.CODI_COMPETICIO = p_codcom
                  AND RES.CODI_EQUIP = p_codequ
                  AND RES.CODI_PILOT = p_codpil
                  AND RES.CODI_COTXE = p_codcot
                  AND RES.POSICIO = 1;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      v_ret      INTEGER;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_NUM_CAR_GUANYADES';
      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      CLOSE c_cars;

      v_ret := v_cars.NUM_CAR_GUANYADES;
      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_NUM_CAR_GUANYADES;

   FUNCTION ACT_NUM_CAR_ACABADES (p_codcom   IN COMPETICIO.CODI%TYPE,
                                  p_codequ   IN EQUIP.CODI%TYPE,
                                  p_codpil   IN PILOT.CODI%TYPE,
                                  p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER
   IS
      -- Si una carrera t la posici a NULL significa que no ha
      -- acabat la carrera.
      CURSOR c_cars
      IS
         SELECT   COUNT ( * ) NUM_CAR_ACABADES
           FROM   RESULTAT RES
          WHERE       RES.CODI_COMPETICIO = p_codcom
                  AND RES.CODI_EQUIP = p_codequ
                  AND RES.CODI_PILOT = p_codpil
                  AND RES.CODI_COTXE = p_codcot
                  AND RES.POSICIO IS NOT NULL;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      v_ret      INTEGER;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_NUM_CAR_ACABADES';
      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      CLOSE c_cars;

      v_ret := v_cars.NUM_CAR_ACABADES;
      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_NUM_CAR_ACABADES;

   -- R12
   FUNCTION ACT_CARBURANT_GASTAT (p_codcom   IN COMPETICIO.CODI%TYPE,
                                  p_codequ   IN EQUIP.CODI%TYPE,
                                  p_codpil   IN PILOT.CODI%TYPE,
                                  p_codcot   IN COTXE.CODI%TYPE)
      RETURN FLOAT
   IS
      CURSOR c_cars
      IS
         SELECT   SUM (TEL.VALOR) CARBURANT_GASTAT
           FROM   RESULTAT RES, TELEMETRIA TEL
          WHERE       RES.CODI = TEL.CODI_RESULTAT
                  AND RES.CODI_EQUIP = TEL.CODI_EQUIP
                  AND RES.CODI_PILOT = TEL.CODI_PILOT
                  AND RES.CODI_COTXE = TEL.CODI_COTXE
                  AND RES.CODI_COMPETICIO = TEL.CODI_COMPETICIO
                  AND RES.CODI_CARRERA = TEL.CODI_CARRERA
                  AND RES.DATA_CARRERA = TEL.DATA_CARRERA
                  AND TEL.CODI_COMPONENT = 'CP04' -- Codi del dipsit de carburant
                  AND TEL.CODI_COMPETICIO = p_codcom
                  AND TEL.CODI_EQUIP = p_codequ
                  AND TEL.CODI_PILOT = p_codpil
                  AND RES.CODI_COTXE = p_codcot;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      v_ret      FLOAT;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_CARBURANT_GASTAT';
      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      IF c_cars%FOUND
      THEN
         v_ret := NVL (v_cars.CARBURANT_GASTAT, 0);
      ELSE
         v_ret := 0;
      END IF;

      CLOSE c_cars;

      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_CARBURANT_GASTAT;

   FUNCTION ACT_VOLUM_TELEMETRIA (p_codcom   IN COMPETICIO.CODI%TYPE,
                                  p_codequ   IN EQUIP.CODI%TYPE,
                                  p_codpil   IN PILOT.CODI%TYPE,
                                  p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER
   IS
      CURSOR c_cars
      IS
         SELECT   COUNT ( * ) VOLUM_TELEMETRIA
           FROM   RESULTAT RES, TELEMETRIA TEL
          WHERE       RES.CODI = TEL.CODI_RESULTAT
                  AND RES.CODI_EQUIP = TEL.CODI_EQUIP
                  AND RES.CODI_PILOT = TEL.CODI_PILOT
                  AND RES.CODI_COTXE = TEL.CODI_COTXE
                  AND RES.CODI_COMPETICIO = TEL.CODI_COMPETICIO
                  AND RES.CODI_CARRERA = TEL.CODI_CARRERA
                  AND RES.DATA_CARRERA = TEL.DATA_CARRERA
                  AND TEL.CODI_COMPETICIO = p_codcom
                  AND TEL.CODI_EQUIP = p_codequ
                  AND TEL.CODI_PILOT = p_codpil
                  AND TEL.CODI_COTXE = p_codcot;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      v_ret      INTEGER;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_VOLUM_TELEMETRIA';
      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      CLOSE c_cars;

      v_ret := NVL (v_cars.VOLUM_TELEMETRIA, 0);
      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_VOLUM_TELEMETRIA;

   PROCEDURE ACT_DINERS_PATROCINADOR (
      p_codcom   IN     COMPETICIO.CODI%TYPE,
      p_codequ   IN     EQUIP.CODI%TYPE,
      p_codpil   IN     PILOT.CODI%TYPE,
      p_codcot   IN     COTXE.CODI%TYPE,
      p_valpat      OUT PATROCINADOR.VALOR_PATROCINI%TYPE,
      p_codpat      OUT PATROCINADOR.CODI%TYPE
   )
   IS
      CURSOR c_cars
      IS
         SELECT   DISTINCT PAT.VALOR_PATROCINI, PAT.CODI
           FROM   PATROCINADOR PAT,
                  EQUIP EQU,
                  FORMACIO FORMA,
                  RESULTAT RES
          WHERE       RES.CODI_EQUIP = FORMA.CODI_EQUIP
                  AND RES.CODI_PILOT = FORMA.CODI_PILOT
                  AND FORMA.CODI_EQUIP = EQU.CODI
                  AND EQU.CODI_PATROCINADOR = PAT.CODI
                  AND RES.CODI_COMPETICIO = p_codcom
                  AND RES.CODI_EQUIP = p_codequ
                  AND RES.CODI_PILOT = p_codpil
                  AND RES.CODI_COTXE = p_codcot;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_no_existeix_pat EXCEPTION;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_DINERS_PATROCINADOR';

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      -- Si exiteix patrocinador
      p_valpat := v_cars.VALOR_PATROCINI;
      p_codpat := v_cars.CODI;

      IF c_cars%FOUND
      THEN
         CLOSE c_cars;

         v_rsp := 'OK';
         UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                    v_log,
                                    v_rsp,
                                    'N');
      ELSE
         CLOSE c_cars;

         -- No te patrocini
         RAISE e_no_existeix_pat;
      END IF;
   EXCEPTION
      WHEN e_no_existeix_pat
      THEN
         v_numerr := 'E015';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         v_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                    v_log,
                                    v_rsp,
                                    'N');
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         v_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, v_rsp);
   END ACT_DINERS_PATROCINADOR;

   FUNCTION ACT_TEMPERATURA_MAX (p_codcom   IN COMPETICIO.CODI%TYPE,
                                 p_codequ   IN EQUIP.CODI%TYPE,
                                 p_codpil   IN PILOT.CODI%TYPE,
                                 p_codcot   IN COTXE.CODI%TYPE)
      RETURN FLOAT
   IS
      CURSOR c_cars
      IS
         SELECT   MAX (TEL.VALOR) TEMPERATURA_MAX
           FROM   RESULTAT RES, TELEMETRIA TEL
          WHERE       RES.CODI = TEL.CODI_RESULTAT
                  AND RES.CODI_EQUIP = TEL.CODI_EQUIP
                  AND RES.CODI_PILOT = TEL.CODI_PILOT
                  AND RES.CODI_COTXE = TEL.CODI_COTXE
                  AND RES.CODI_COMPETICIO = TEL.CODI_COMPETICIO
                  AND RES.CODI_CARRERA = TEL.CODI_CARRERA
                  AND RES.DATA_CARRERA = TEL.DATA_CARRERA
                  AND TEL.CODI_COMPONENT = 'CP05' -- Codi del component de termmetre
                  AND TEL.CODI_COMPETICIO = p_codcom
                  AND TEL.CODI_EQUIP = p_codequ
                  AND TEL.CODI_PILOT = p_codpil
                  AND RES.CODI_COTXE = p_codcot;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      v_ret      FLOAT;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_TEMPERATURA_MAX';
      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      IF c_cars%FOUND
      THEN
         v_ret := v_cars.TEMPERATURA_MAX;
      ELSE
         v_ret := NULL;
      END IF;

      CLOSE c_cars;

      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_TEMPERATURA_MAX;

   FUNCTION ACT_PER_COM_DEFECTUOS (p_codcom   IN COMPETICIO.CODI%TYPE,
                                   p_codequ   IN EQUIP.CODI%TYPE,
                                   p_codpil   IN PILOT.CODI%TYPE,
                                   p_codcot   IN COTXE.CODI%TYPE)
      RETURN FLOAT
   IS
      CURSOR c_cars
      IS
         SELECT   COUNT ( * ) VOLUM_TELEMETRIA
           FROM   RESULTAT RES, TELEMETRIA TEL
          WHERE       RES.CODI = TEL.CODI_RESULTAT
                  AND RES.CODI_EQUIP = TEL.CODI_EQUIP
                  AND RES.CODI_PILOT = TEL.CODI_PILOT
                  AND RES.CODI_COTXE = TEL.CODI_COTXE
                  AND RES.CODI_COMPETICIO = TEL.CODI_COMPETICIO
                  AND RES.CODI_CARRERA = TEL.CODI_CARRERA
                  AND RES.DATA_CARRERA = TEL.DATA_CARRERA
                  AND TEL.CODI_COMPETICIO = p_codcom
                  AND TEL.CODI_EQUIP = p_codequ
                  AND TEL.CODI_PILOT = p_codpil
                  AND TEL.CODI_COTXE = p_codcot;

      v_cars      c_cars%ROWTYPE;

      -- Cursor que extreu el nombre de components defectuosos. Un component s defectus
      -- quan la seva dada telemtrica associada t el camp VALOR a NULL.
      CURSOR c_com_def
      IS
         SELECT   COUNT ( * ) COM_DEFECTUOSOS
           FROM   RESULTAT RES, TELEMETRIA TEL
          WHERE       RES.CODI = TEL.CODI_RESULTAT
                  AND RES.CODI_EQUIP = TEL.CODI_EQUIP
                  AND RES.CODI_PILOT = TEL.CODI_PILOT
                  AND RES.CODI_COTXE = TEL.CODI_COTXE
                  AND RES.CODI_COMPETICIO = TEL.CODI_COMPETICIO
                  AND RES.CODI_CARRERA = TEL.CODI_CARRERA
                  AND RES.DATA_CARRERA = TEL.DATA_CARRERA
                  AND TEL.VALOR IS NULL
                  AND TEL.CODI_COMPETICIO = p_codcom
                  AND TEL.CODI_EQUIP = p_codequ
                  AND TEL.CODI_PILOT = p_codpil
                  AND TEL.CODI_COTXE = p_codcot;

      v_com_def   c_com_def%ROWTYPE;
      -- Variables de LOG
      v_nompro    LOGS.NOM_PROCEDIMENT%TYPE;
      v_log       LOGS.LOG%TYPE;
      v_rsp       LOGS.RSP%TYPE;
      v_ret       FLOAT;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_PER_COM_DEFECTUOS';
      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      OPEN c_com_def;

      FETCH c_com_def INTO   v_com_def;

      CLOSE c_com_def;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      CLOSE c_cars;

      -- Evitar la divisi 0/0
      IF v_com_def.COM_DEFECTUOSOS = 0 AND v_cars.VOLUM_TELEMETRIA = 0
      THEN
         v_ret := 0;
      ELSE
         v_ret := (100 * v_com_def.COM_DEFECTUOSOS) / v_cars.VOLUM_TELEMETRIA;
      END IF;

      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_PER_COM_DEFECTUOS;

   FUNCTION ACT_CLASSIFICACIO (p_codcom   IN COMPETICIO.CODI%TYPE,
                               p_codequ   IN EQUIP.CODI%TYPE,
                               p_codpil   IN PILOT.CODI%TYPE,
                               p_codcot   IN COTXE.CODI%TYPE)
      RETURN INTEGER
   IS
      CURSOR c_cars
      IS
         SELECT   SUM (RES.PUNTUACIO) CLASSIFICACIO
           FROM   RESULTAT RES
          WHERE       RES.CODI_COMPETICIO = p_codcom
                  AND RES.CODI_EQUIP = p_codequ
                  AND RES.CODI_PILOT = p_codpil
                  AND RES.CODI_COTXE = p_codcot;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      v_ret      INTEGER;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_CLASSIFICACIO';
      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codequ ->'
         || p_codequ
         || ',p_codpil ->'
         || p_codpil
         || ',p_codcot ->'
         || p_codcot;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      IF c_cars%FOUND
      THEN
         v_ret := NVL (v_cars.CLASSIFICACIO, 0);
      ELSE
         v_ret := 0;
      END IF;

      CLOSE c_cars;

      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_CLASSIFICACIO;

   PROCEDURE ACT_VOLTA_RAPIDA (p_codcom   IN     COMPETICIO.CODI%TYPE,
                               p_codcar   IN     CARRERA.CODI%TYPE,
                               p_volrap      OUT RESULTAT.VOLTA_RAPIDA%TYPE,
                               p_datcar      OUT CARRERA.DATA_CARRERA%TYPE,
                               p_codpil      OUT PILOT.CODI%TYPE,
                               p_codequ      OUT PILOT.CODI%TYPE)
   IS
      CURSOR c_volrap
      IS
         SELECT   RES.VOLTA_RAPIDA,
                  RES.DATA_CARRERA,
                  RES.CODI_PILOT,
                  RES.CODI_EQUIP
           FROM   RESULTAT RES
          WHERE   RES.VOLTA_RAPIDA =
                     (SELECT   MIN (RES.VOLTA_RAPIDA)
                        FROM   RESULTAT RES
                       WHERE   RES.CODI_COMPETICIO = p_codcom
                               AND RES.CODI_CARRERA = p_codcar);

      v_volrap   c_volrap%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_no_existeix_volrap EXCEPTION;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_VOLTA_RAPIDA';

      OPEN c_volrap;

      FETCH c_volrap INTO   v_volrap;

      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codcar ->'
         || p_codcar
         || ',p_codcar ->'
         || p_codcar
         || ',p_volrap ->'
         || v_volrap.VOLTA_RAPIDA
         || ',p_datcar ->'
         || v_volrap.DATA_CARRERA
         || ',p_codpil ->'
         || v_volrap.CODI_PILOT
         || ',p_codequ ->'
         || v_volrap.CODI_EQUIP;

      -- Si la carrera t volta rpida
      p_volrap := v_volrap.VOLTA_RAPIDA;
      p_datcar := v_volrap.DATA_CARRERA;
      p_codpil := v_volrap.CODI_PILOT;
      p_codequ := v_volrap.CODI_EQUIP;

      IF c_volrap%FOUND
      THEN
         CLOSE c_volrap;

         v_rsp := 'OK';
         UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                    v_log,
                                    v_rsp,
                                    'N');
      ELSE
         CLOSE c_volrap;

         -- Aquesta carrera no t volta rpida
         RAISE e_no_existeix_volrap;
      END IF;
   EXCEPTION
      WHEN e_no_existeix_volrap
      THEN
         v_numerr := 'E005';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         v_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                    v_log,
                                    v_rsp,
                                    'N');
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         v_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, v_rsp);
   END ACT_VOLTA_RAPIDA;

   PROCEDURE ACT_TEMP_MAXIMA (p_codcom   IN     COMPETICIO.CODI%TYPE,
                              p_codcar   IN     CARRERA.CODI%TYPE,
                              p_codcot   IN     COTXE.CODI%TYPE,
                              p_datcar      OUT CARRERA.DATA_CARRERA%TYPE,
                              p_codpil      OUT PILOT.CODI%TYPE,
                              p_temmax      OUT TELEMETRIA.VALOR%TYPE)
   IS
      -- Cursor que extreu el valor de temperatura mxim del component termmetre (codi CP05)
      -- Es considera que els dos membres de l'equip condueixen un mateix codi de cotxe.
      -- Si existeixen dos registres amb igual temperatura mxima s'escolleix el primer de la consulta.

      CURSOR c_temmax
      IS
         SELECT   TEL.DATA_CARRERA, TEL.CODI_PILOT, TEL.VALOR
           FROM   TELEMETRIA TEL
          WHERE   TEL.VALOR =
                     (SELECT   MAX (TEL.VALOR)
                        FROM   TELEMETRIA TEL
                       WHERE       TEL.CODI_COMPONENT = 'CP05' -- Codi del component termmetre que t dades de temperatura.
                               AND TEL.CODI_COMPETICIO = p_codcom
                               AND TEL.CODI_CARRERA = p_codcar
                               AND TEL.CODI_COTXE = p_codcot)
                  AND TEL.CODI_COMPETICIO = p_codcom
                  AND TEL.CODI_CARRERA = p_codcar
                  AND TEL.CODI_COTXE = p_codcot;

      v_temmax   c_temmax%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_no_existeix_temmax EXCEPTION;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_TEMP_MAXIMA';

      OPEN c_temmax;

      FETCH c_temmax INTO   v_temmax;

      v_log :=
            'p_codcom  ->'
         || p_codcom
         || ',p_codcar ->'
         || p_codcar
         || ',p_codcot ->'
         || p_codcot
         || ',p_datcar ->'
         || v_temmax.DATA_CARRERA
         || ',p_codpil ->'
         || v_temmax.CODI_PILOT
         || ',p_temmax ->'
         || v_temmax.VALOR;

      -- Si la cursa te valor mxim de temperatura
      p_datcar := v_temmax.DATA_CARRERA;
      p_codpil := v_temmax.CODI_PILOT;
      p_temmax := v_temmax.VALOR;

      IF c_temmax%FOUND
      THEN
         CLOSE c_temmax;

         v_rsp := 'OK';
         UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                    v_log,
                                    v_rsp,
                                    'N');
      ELSE
         CLOSE c_temmax;

         -- Aquesta carrera no t volta rpida
         RAISE e_no_existeix_temmax;
      END IF;
   EXCEPTION
      WHEN e_no_existeix_temmax
      THEN
         v_numerr := 'E006';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         v_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                    v_log,
                                    v_rsp,
                                    'N');
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         v_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, v_rsp);
   END ACT_TEMP_MAXIMA;

   FUNCTION ACT_NUM_COMPONENTS (p_codcom   IN COMPETICIO.CODI%TYPE,
                                p_codfab   IN FABRICANT.CODI%TYPE)
      RETURN INTEGER
   IS
      -- Cursor que extreu el nombre de components en competici d'un fabricant
      CURSOR c_cars
      IS
         SELECT   COUNT ( * ) NUM_COMP
           FROM   TELEMETRIA TEL, COMPONENT COM
          WHERE       TEL.CODI_COMPONENT = COM.CODI
                  AND TEL.CODI_COMPETICIO = p_codcom
                  AND COM.CODI_FABRICANT = p_codfab;

      v_cars     c_cars%ROWTYPE;
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      v_ret      INTEGER;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_NUM_COMPONENTS';
      v_log := 'p_codcom  ->' || p_codcom || ',p_codfab ->' || p_codfab;

      OPEN c_cars;

      FETCH c_cars INTO   v_cars;

      CLOSE c_cars;

      v_ret := v_cars.NUM_COMP;
      v_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro,
                                 v_log || ' RETURN ->' || v_ret,
                                 v_rsp,
                                 'N');
      RETURN v_ret;
   END ACT_NUM_COMPONENTS;

   PROCEDURE ACT_EST_VOLTES_RAPIDES (
      p_est   IN     EST_VOLTES_RAPIDES%ROWTYPE,
      p_rsp      OUT LOGS.RSP%TYPE
   )
   IS
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      v_codest   EST_VOLTES_RAPIDES.CODI%TYPE;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_EST_VOLTES_RAPIDES';
      v_log :=
            'p_est.CODI -> '
         || p_est.CODI
         || ',p_est.CODI_COMPETICIO ->'
         || p_est.CODI_COMPETICIO
         || ',p_est.CODI_CARRERA -> '
         || p_est.CODI_CARRERA;

      -- Verificar NULLS dels parmetres obligatoris
      IF p_est.CODI_CARRERA IS NULL OR p_est.CODI_COMPETICIO IS NULL
      THEN
         RAISE e_null;
      END IF;

      -- Comprovar si el registre existeix dintre de EST_VOLTES_RAPIDES
      v_codest :=
         UTILITATS_PKG.OBTENIR_EST_VOLTES_RAPIDES (p_est.CODI_COMPETICIO,
                                                   p_est.CODI_CARRERA);

      IF v_codest IS NULL
      THEN
         DBMS_OUTPUT.put_line ('INSERT EST_VOLTES_RAPIDES');

         -- Si no existeix el registre dintre EST_VOLTES_RAPIDES
         -- fer INSERT
         INSERT INTO EST_VOLTES_RAPIDES (CODI_COMPETICIO, CODI_CARRERA)
           VALUES   (p_est.CODI_COMPETICIO, p_est.CODI_CARRERA);
      ELSE
         DBMS_OUTPUT.put_line ('UPDATE EST_VOLTES_RAPIDES');

         -- Si existeix, ACTUALITAR EL REGISTRE
         UPDATE   EST_VOLTES_RAPIDES
            SET   CODI_COMPETICIO = p_est.CODI_COMPETICIO,
                  CODI_CARRERA = p_est.CODI_CARRERA
          WHERE   CODI = v_codest;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END ACT_EST_VOLTES_RAPIDES;

   PROCEDURE ACT_EST_TEMP_MAXIMES (p_est   IN     EST_TEMP_MAXIMES%ROWTYPE,
                                   p_rsp      OUT LOGS.RSP%TYPE)
   IS
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      v_codest   EST_VOLTES_RAPIDES.CODI%TYPE;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_EST_TEMP_MAXIMES ';
      v_log :=
            'p_est.CODI ->'
         || p_est.CODI
         || ',p_est.CODI_COMPETICIO -> '
         || p_est.CODI_COMPETICIO
         || ',p_est.CODI_CARRERA -> '
         || p_est.CODI_CARRERA
         || ',p_est.CODI_COTXE -> '
         || p_est.CODI_COTXE;

      -- Verificar NULLS dels parmetres obligatoris
      IF    p_est.CODI_CARRERA IS NULL
         OR p_est.CODI_COMPETICIO IS NULL
         OR p_est.CODI_COTXE IS NULL
      THEN
         RAISE e_null;
      END IF;

      -- Comprovar si el registre existeix dintre de EST_TEMP_MAXIMES
      v_codest :=
         UTILITATS_PKG.OBTENIR_EST_TEMP_MAXIMES (p_est.CODI_COMPETICIO,
                                                 p_est.CODI_CARRERA,
                                                 p_est.CODI_COTXE);

      IF v_codest IS NULL
      THEN
         -- Si no existeix el registre dintre EST_VOLTES_RAPIDES
         -- fer INSERT
         INSERT INTO EST_TEMP_MAXIMES (
                                          CODI_COMPETICIO,
                                          CODI_CARRERA,
                                          CODI_COTXE
                    )
           VALUES   (
                        p_est.CODI_COMPETICIO,
                        p_est.CODI_CARRERA,
                        p_est.CODI_COTXE
                    );
      ELSE
         -- Si existeix, ACTUALITAR EL REGISTRE
         UPDATE   EST_TEMP_MAXIMES
            SET   CODI_COMPETICIO = p_est.CODI_COMPETICIO,
                  CODI_CARRERA = p_est.CODI_CARRERA,
                  CODI_COTXE = p_est.CODI_COTXE
          WHERE   CODI = v_codest;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END ACT_EST_TEMP_MAXIMES;

   PROCEDURE ACT_EST_COM_PIL (p_est   IN     EST_COM_PIL%ROWTYPE,
                              p_rsp      OUT LOGS.RSP%TYPE)
   IS
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      v_codest   EST_VOLTES_RAPIDES.CODI%TYPE;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_EST_COM_PIL';
      v_log :=
            'p_est.CODI ->'
         || p_est.CODI
         || ',p_est.CODI_COMPETICIO -> '
         || p_est.CODI_COMPETICIO
         || ',p_est.INICI_COMPETICIO -> '
         || p_est.INICI_COMPETICIO
         || ',p_est.FINAL_COMPETICIO -> '
         || p_est.FINAL_COMPETICIO
         || ',p_est.ANY_COMPETICIO -> '
         || p_est.ANY_COMPETICIO
         || ',p_est.CODI_EQUIP-> '
         || p_est.CODI_EQUIP
         || ',p_est.CODI_PILOT-> '
         || p_est.CODI_PILOT
         || ',p_est.CODI_COTXE -> '
         || p_est.CODI_COTXE;

      -- Verificar NULLS dels parmetres obligatoris
      IF    p_est.CODI_COMPETICIO IS NULL
         OR p_est.ANY_COMPETICIO IS NULL
         OR p_est.CODI_EQUIP IS NULL
         OR p_est.CODI_PILOT IS NULL
         OR p_est.CODI_COTXE IS NULL
      THEN
         RAISE e_null;
      END IF;

      -- Comprovar si el registre existeix dintre de EST_COM_PIL
      v_codest :=
         UTILITATS_PKG.OBTENIR_EST_COM_PIL (p_est.ANY_COMPETICIO,
                                            p_est.CODI_COMPETICIO,
                                            p_est.CODI_EQUIP,
                                            p_est.CODI_PILOT,
                                            p_est.CODI_COTXE);

      IF v_codest IS NULL
      THEN
         -- Si no existeix el registre dintre EST_COM_PIL
         -- fer INSERT
         INSERT INTO EST_COM_PIL (ANY_COMPETICIO,
                                  INICI_COMPETICIO,
                                  FINAL_COMPETICIO,
                                  CODI_COMPETICIO,
                                  CODI_EQUIP,
                                  CODI_PILOT,
                                  CODI_COTXE)
           VALUES   (p_est.ANY_COMPETICIO,
                     p_est.INICI_COMPETICIO,
                     p_est.FINAL_COMPETICIO,
                     p_est.CODI_COMPETICIO,
                     p_est.CODI_EQUIP,
                     p_est.CODI_PILOT,
                     p_est.CODI_COTXE);
      ELSE
         -- Si existeix, ACTUALITAR EL REGISTRE EST_COM_PIL
         UPDATE   EST_COM_PIL
            SET   ANY_COMPETICIO = p_est.ANY_COMPETICIO,
                  INICI_COMPETICIO = p_est.INICI_COMPETICIO,
                  FINAL_COMPETICIO = p_est.FINAL_COMPETICIO,
                  CODI_COMPETICIO = p_est.CODI_COMPETICIO,
                  CODI_EQUIP = p_est.CODI_EQUIP,
                  CODI_PILOT = p_est.CODI_PILOT,
                  CODI_COTXE = p_est.CODI_COTXE
          WHERE   CODI = v_codest;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END ACT_EST_COM_PIL;

   PROCEDURE ACT_EST_NUM_COMP_FABRICANT (
      p_est   IN     EST_NUM_COMP_FABRICANT%ROWTYPE,
      p_rsp      OUT LOGS.RSP%TYPE
   )
   IS
      -- Variables de LOG
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_rsp      LOGS.RSP%TYPE;
      -- Variables pel control d'errors i excepcions
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      v_codest   EST_VOLTES_RAPIDES.CODI%TYPE;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'ACT_EST_NUM_COMP_FABRICANT';
      v_log :=
            'p_est.CODI ->'
         || p_est.CODI
         || ',p_est.CODI_COMPETICIO -> '
         || p_est.CODI_COMPETICIO
         || ',p_est.CODI_FABRICANT -> '
         || p_est.CODI_FABRICANT;

      -- Verificar NULLS dels parmetres obligatoris
      IF p_est.CODI_COMPETICIO IS NULL OR p_est.CODI_FABRICANT IS NULL
      THEN
         RAISE e_null;
      END IF;

      -- Comprovar si el registre existeix dintre de EST_NUM_COMP_FABRICANT
      v_codest :=
         UTILITATS_PKG.OBTENIR_EST_NUM_COMP_FABRICANT (p_est.CODI_COMPETICIO,
                                                       p_est.CODI_FABRICANT);

      IF v_codest IS NULL
      THEN
         DBMS_OUTPUT.put_line ('INSERT EST_NUM_COMP_FABRICANT');

         -- Si no existeix el registre dintre EST_NUM_COMP_FABRICANT
         -- fer INSERT
         INSERT INTO EST_NUM_COMP_FABRICANT (CODI_COMPETICIO, CODI_FABRICANT)
           VALUES   (p_est.CODI_COMPETICIO, p_est.CODI_FABRICANT);
      ELSE
         DBMS_OUTPUT.put_line ('UPDATE EST_NUM_COMP_FABRICANT');

         -- Si existeix, ACTUALITAR EL REGISTRE EST_NUM_COMP_FABRICANT
         UPDATE   EST_NUM_COMP_FABRICANT
            SET   CODI_COMPETICIO = p_est.CODI_COMPETICIO,
                  CODI_FABRICANT = p_est.CODI_FABRICANT
          WHERE   CODI = v_codest;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END ACT_EST_NUM_COMP_FABRICANT;
END ESTADISTICA_PKG;
/

CREATE OR REPLACE PACKAGE GSIMONET.TELEMETRIA_PKG
AS
   /******************************************************************************
     NOM:       TELEMETRIA_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que l'actor
                Sensor de dades telemtriques pot executar des de l'aplicaci superior.
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Inserta una dada telemtrica a la BD.
    *  Parmetres d'entrada
    *  p_codequ : Codi de l'equip
    *  p_codpil : Codi del pilot
    *  p_codcot : Cdi del cotxe
    *  p_codcom : Codi de la competici
    *  p_codcar : Codi de la carrera
    *  p_datcar : Data de la carrera
    *  p_codcpt : Codi del component
    *  p_moment : Hora de la dada telemtrica
    *  p_ambit  : mbit de la dada telemtrica.
    *  p_valor  : Valor de la dada telemtrica.
    *  p_unimes : Unitat de mesura de la dada telemtrica.
    *  Parmetres de sortida
    *  p_codtel : Codi de la dada telemtrica generada per la seqencia
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/

   PROCEDURE INSERT_TELEMETRIA (
      p_codequ   IN     TELEMETRIA.CODI_EQUIP%TYPE,
      p_codpil   IN     TELEMETRIA.CODI_PILOT%TYPE,
      p_codcot   IN     TELEMETRIA.CODI_COTXE%TYPE,
      p_codcom   IN     TELEMETRIA.CODI_COMPETICIO%TYPE,
      p_codcar   IN     TELEMETRIA.CODI_CARRERA%TYPE,
      p_datcar   IN     TELEMETRIA.DATA_CARRERA%TYPE,
      p_codcpt   IN     TELEMETRIA.CODI_COMPONENT%TYPE,
      p_moment   IN     TELEMETRIA.MOMENT%TYPE,
      p_ambit    IN     TELEMETRIA.AMBIT%TYPE,
      p_valor    IN     TELEMETRIA.VALOR%TYPE DEFAULT NULL ,
      p_unimes   IN     TELEMETRIA.UNITAT_MESURA%TYPE,
      p_codtel      OUT RESULTAT.CODI%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   );

   /****************************************************************************
      *  Descripci: Elimina una dada telemtrica a la BD.
   *  Parmetres d'entrada
   *  p_codtel : Codi de la dada telemtrica generada per la seqencia
   *  Parmetres de sortida
   *  p_rsp:  'OK' en cas de execuci correcta
   *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
   *
   *  Autor            Data         Descripci
   *------------    ------------    ------------------------
   * GSIMONET        02-04-2015   Creaci del procediment
   ****************************************************************************/
   PROCEDURE DELETE_TELEMETRIA (p_codtel   IN     TELEMETRIA.CODI%TYPE,
                                p_rsp         OUT LOGS.RSP%TYPE);
END TELEMETRIA_PKG;
/

CREATE OR REPLACE PACKAGE BODY GSIMONET.TELEMETRIA_PKG
AS
   /******************************************************************************
     NOM:       TELEMETRIA_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que l'actor
                Sensor de dades telemtriques pot executar des de l'aplicaci superior.
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Inserta una dada telemtrica a la BD.
    *  Parmetres d'entrada
    *  p_codequ : Codi de l'equip
    *  p_codpil : Codi del pilot
    *  p_codcot : Cdi del cotxe
    *  p_codcom : Codi de la competici
    *  p_codcar : Codi de la carrera
    *  p_datcar : Data de la carrera
    *  p_codcpt : Codi del component
    *  p_moment : Hora de la dada telemtrica
    *  p_ambit  : mbit de la dada telemtrica.
    *  p_valor  : Valor de la dada telemtrica.
    *  p_unimes : Unitat de mesura de la dada telemtrica.
    *  Parmetres de sortida
    *  p_codtel : Codi de la dada telemtrica generada per la seqencia
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/

   PROCEDURE INSERT_TELEMETRIA (
      p_codequ   IN     TELEMETRIA.CODI_EQUIP%TYPE,
      p_codpil   IN     TELEMETRIA.CODI_PILOT%TYPE,
      p_codcot   IN     TELEMETRIA.CODI_COTXE%TYPE,
      p_codcom   IN     TELEMETRIA.CODI_COMPETICIO%TYPE,
      p_codcar   IN     TELEMETRIA.CODI_CARRERA%TYPE,
      p_datcar   IN     TELEMETRIA.DATA_CARRERA%TYPE,
      p_codcpt   IN     TELEMETRIA.CODI_COMPONENT%TYPE,
      p_moment   IN     TELEMETRIA.MOMENT%TYPE,
      p_ambit    IN     TELEMETRIA.AMBIT%TYPE,
      p_valor    IN     TELEMETRIA.VALOR%TYPE DEFAULT NULL ,
      p_unimes   IN     TELEMETRIA.UNITAT_MESURA%TYPE,
      p_codtel      OUT RESULTAT.CODI%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   )
   IS
      -- Variables pel control d'errors i excepcions
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      e_no_formacio EXCEPTION;
      e_no_cotxe EXCEPTION;
      e_no_carrera EXCEPTION;
      e_no_component EXCEPTION;
      e_dates_distintes EXCEPTION;
      e_no_estadistica EXCEPTION;
      e_no_resultat EXCEPTION;

      CURSOR c_dates
      IS
         SELECT   COM.DATA_INICI, COM.DATA_FI
           FROM   COMPETICIO COM
          WHERE   COM.CODI = p_codcom;

      v_dates    c_dates%ROWTYPE;
      v_tel      TELEMETRIA%ROWTYPE;
      v_est      EST_NUM_COMP_FABRICANT%ROWTYPE;
      v_est2     EST_TEMP_MAXIMES%ROWTYPE;
      v_est3     EST_COM_PIL%ROWTYPE;
      v_res      RESULTAT%ROWTYPE;
   BEGIN
      p_codtel := NULL;
      -- Parmetres de LOG
      v_nompro := 'INSERT_TELEMETRIA';
      v_log :=
            'p_codequ  -> '
         || p_codequ
         || 'p_codpil  -> '
         || p_codpil
         || 'p_codcot  -> '
         || p_codcot
         || 'p_codcom  -> '
         || p_codcom
         || 'p_codcar  -> '
         || p_codcar
         || 'p_datcar  -> '
         || p_datcar
         || 'p_codcpt  -> '
         || p_codcpt
         || 'p_moment  -> '
         || p_moment
         || 'p_ambit  -> '
         || p_ambit
         || 'p_valor  -> '
         || p_valor
         || 'p_unimes  -> '
         || p_unimes;

      -- Comprovar parmetre obligatoris
      IF    p_codequ IS NULL
         OR p_codpil IS NULL
         OR p_codcot IS NULL
         OR p_codcom IS NULL
         OR p_codcar IS NULL
         OR p_datcar IS NULL
         OR p_codcpt IS NULL
         OR p_moment IS NULL
         OR p_ambit IS NULL
         OR p_unimes IS NULL
      THEN
         RAISE e_null;
      END IF;

      -- Comprovar que la formaci existeix
      IF UTILITATS_PKG.EXISTEIX_FORMACIO (p_codpil,
                                          p_codequ,
                                          EXTRACT (YEAR FROM p_datcar)) = 'N'
      THEN
         RAISE e_no_formacio;
      END IF;

      -- Comprovar que el cotxe existeix
      IF UTILITATS_PKG.EXISTEIX_COTXE (p_codcot, p_codequ) = 'N'
      THEN
         RAISE e_no_cotxe;
      END IF;

      -- Comprovar que la competicio t associada la carrera
      IF UTILITATS_PKG.EXISTEIX_CARRERA (p_codcom, p_codcar) = 'N'
      THEN
         RAISE e_no_carrera;
      END IF;

      -- Comprovar que existeix el component
      IF UTILITATS_PKG.EXISTEIX_COMPONENT (p_codcpt) = 'N'
      THEN
         RAISE e_no_component;
      END IF;

      -- Comprovar que la data de la carrera = data del moment
      IF p_datcar <> (TO_DATE (TO_CHAR (p_moment, 'DD/MM/YYYY')))
      THEN
         RAISE e_dates_distintes;
      END IF;

      -- Recuperar la row de RESULTATS
      v_res :=
         UTILITATS_PKG.OBTENIR_RESULTAT (p_codequ,
                                         p_codpil,
                                         p_codcot,
                                         p_codcom,
                                         p_codcar,
                                         p_datcar);

      IF v_res.CODI IS NULL
      THEN
         RAISE e_no_resultat;
      END IF;

      -- Generar codi de TELEMETRIA usant la seqncia
      SELECT   GSIMONET.TELEMETRIA_SQ.NEXTVAL INTO p_codtel FROM DUAL;

      -- Insertar dades a TELEMETRIA
      INSERT INTO TELEMETRIA (CODI,
                              CODI_RESULTAT,
                              CODI_EQUIP,
                              CODI_PILOT,
                              CODI_COTXE,
                              CODI_COMPETICIO,
                              CODI_CARRERA,
                              DATA_CARRERA,
                              CODI_COMPONENT,
                              MOMENT,
                              AMBIT,
                              VALOR,
                              UNITAT_MESURA)
        VALUES   (p_codtel,
                  v_res.CODI,
                  p_codequ,
                  p_codpil,
                  p_codcot,
                  p_codcom,
                  p_codcar,
                  p_datcar,
                  p_codcpt,
                  p_moment,
                  p_ambit,
                  p_valor,
                  p_unimes);

      -- Actualitza EST_NUM_COMP_FABRICANT
      v_est.CODI_COMPETICIO := p_codcom;
      v_est.CODI_FABRICANT := UTILITATS_PKG.OBTENIR_FABRICANT (p_codcpt);

      ESTADISTICA_PKG.ACT_EST_NUM_COMP_FABRICANT (v_est, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      -- Actualitza EST_TEMP_MAXIMES
      v_est2.CODI_COMPETICIO := p_codcom;
      v_est2.CODI_CARRERA := p_codcar;
      v_est2.CODI_COTXE := p_codcot;

      ESTADISTICA_PKG.ACT_EST_TEMP_MAXIMES (v_est2, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      -- Actualitza EST_COM_PIL
      OPEN c_dates;

      FETCH c_dates INTO   v_dates;

      CLOSE c_dates;

      v_est3.INICI_COMPETICIO := v_dates.DATA_INICI;
      v_est3.FINAL_COMPETICIO := v_dates.DATA_FI;
      v_est3.ANY_COMPETICIO := TO_NUMBER (EXTRACT (YEAR FROM p_datcar));
      v_est3.CODI_COMPETICIO := p_codcom;
      v_est3.CODI_EQUIP := p_codequ;
      v_est3.CODI_PILOT := p_codpil;
      v_est3.CODI_COTXE := p_codcot;

      ESTADISTICA_PKG.ACT_EST_COM_PIL (v_est3, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      COMMIT;
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_formacio
      THEN
         v_numerr := 'E007';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_cotxe
      THEN
         v_numerr := 'E008';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_carrera
      THEN
         v_numerr := 'E009';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_component
      THEN
         v_numerr := 'E013';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_dates_distintes
      THEN
         v_numerr := 'E012';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_estadistica
      THEN
         v_numerr := 'E011';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_resultat
      THEN
         v_numerr := 'E014';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END INSERT_TELEMETRIA;

   /****************************************************************************
          *  Descripci: Elimina una dada telemtrica a la BD.
    *  Parmetres d'entrada
    *  p_codtel : Codi de la dada telemtrica generada per la seqencia
    *  Parmetres de sortida
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci del procediment
    ****************************************************************************/

   PROCEDURE DELETE_TELEMETRIA (p_codtel   IN     TELEMETRIA.CODI%TYPE,
                                p_rsp         OUT LOGS.RSP%TYPE)
   IS
      -- Variables pel control d'errors i excepcions
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      e_no_telemetria EXCEPTION;
      e_no_estadistica EXCEPTION;
      v_est      EST_NUM_COMP_FABRICANT%ROWTYPE;
      v_est2     EST_TEMP_MAXIMES%ROWTYPE;
      v_est3     EST_COM_PIL%ROWTYPE;
      v_tel      TELEMETRIA%ROWTYPE;

      CURSOR c_dates (p_codcom IN COMPETICIO.CODI%TYPE)
      IS
         SELECT   COM.DATA_INICI, COM.DATA_FI
           FROM   COMPETICIO COM
          WHERE   COM.CODI = p_codcom;

      v_dates    c_dates%ROWTYPE;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'DELETE_TELEMETRIA';
      v_log := 'p_codtel  -> ' || p_codtel;

      -- Comprovar parmetre obligatoris
      IF p_codtel IS NULL
      THEN
         RAISE e_null;
      END IF;

      v_tel := UTILITATS_PKG.OBTENIR_TELEMETRIA (p_codtel);

      -- Comprovar que la telemetria existeix
      IF (v_tel.CODI = NULL)
      THEN
         RAISE e_no_telemetria;
      END IF;

      -- Eliminar la row de la telemetria
      DELETE   TELEMETRIA TEL
       WHERE   TEL.CODI = p_codtel;

      -- Actualitza EST_NUM_COMP_FABRICANT
      v_est.CODI_COMPETICIO := v_tel.CODI_COMPETICIO;
      v_est.CODI_FABRICANT :=
         UTILITATS_PKG.OBTENIR_FABRICANT (v_tel.CODI_COMPONENT);

      ESTADISTICA_PKG.ACT_EST_NUM_COMP_FABRICANT (v_est, p_rsp);

      DBMS_OUTPUT.put_line (p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      -- Actualitza EST_TEMP_MAXIMES
      v_est2.CODI_COMPETICIO := v_tel.CODI_COMPETICIO;
      v_est2.CODI_CARRERA := v_tel.CODI_CARRERA;
      v_est2.CODI_COTXE := v_tel.CODI_COTXE;

      ESTADISTICA_PKG.ACT_EST_TEMP_MAXIMES (v_est2, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

    OPEN c_dates(v_tel.CODI_COMPETICIO);

      FETCH c_dates INTO   v_dates;

      CLOSE c_dates;

      -- Actualitza EST_COM_PIL
      v_est3.ANY_COMPETICIO :=
         TO_NUMBER (EXTRACT (YEAR FROM v_tel.DATA_CARRERA));
      v_est3.INICI_COMPETICIO := v_dates.DATA_INICI;
      v_est3.FINAL_COMPETICIO := v_dates.DATA_FI;
      v_est3.CODI_COMPETICIO := v_tel.CODI_COMPETICIO;
      v_est3.CODI_EQUIP := v_tel.CODI_EQUIP;
      v_est3.CODI_PILOT := v_tel.CODI_PILOT;
      v_est3.CODI_COTXE := v_tel.CODI_COTXE;

      ESTADISTICA_PKG.ACT_EST_COM_PIL (v_est3, p_rsp);
      DBMS_OUTPUT.put_line (p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      COMMIT;
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_telemetria
      THEN
         v_numerr := 'E016';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_estadistica
      THEN
         v_numerr := 'E011';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END DELETE_TELEMETRIA;
END TELEMETRIA_PKG;
/

CREATE OR REPLACE PACKAGE GSIMONET.FIA_PKG
AS
   /******************************************************************************
     NOM:       FIA_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que l'actor
                Administrador FIA pot executar des de l'aplicaci superior.
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Inserta/actualitza un resultat a la BD.
    *  Parmetros d'entrada
    *  p_codequ : Codi de l'equip
    *  p_codpil : Codi del pilot
    *  p_codcot : Cdi del cotxe
    *  p_codcom : Codi de la competici
    *  p_codcar : Codi de la carrera
    *  p_datcar : Data de la carrera
    *  p_codpos : Posici del pilot a la carrera.
    *  p_volrap : Volta rpida del pilot a la carrera
    *  Parmetros de sortida
    *  p_codres : Codi del resultat generat per la seqencia
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   PROCEDURE INSERT_UPDATE_RESULTAT (
      p_codequ   IN     RESULTAT.CODI_EQUIP%TYPE,
      p_codpil   IN     RESULTAT.CODI_PILOT%TYPE,
      p_codcot   IN     RESULTAT.CODI_COTXE%TYPE,
      p_codcom   IN     RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN     RESULTAT.CODI_CARRERA%TYPE,
      p_datcar   IN     RESULTAT.DATA_CARRERA%TYPE,
      p_codpos   IN     RESULTAT.POSICIO%TYPE,
      p_volrap   IN     RESULTAT.VOLTA_RAPIDA%TYPE,
      p_codres      OUT RESULTAT.CODI%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   );

   /* ***************************************************************************
       *  Descripci: Elimina un resultat de la BD.
    *  Parmetros d'entrada
    *  p_codequ : Codi de l'equip
    *  p_codpil : Codi del pilot
    *  p_codcot : Cdi del cotxe
    *  p_codcar : Codi de la carrera
    *  p_datcar : Data de la carrera
    *  Parmetros de sortida
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   PROCEDURE DELETE_RESULTAT (
      p_codequ   IN     RESULTAT.CODI_EQUIP%TYPE,
      p_codpil   IN     RESULTAT.CODI_PILOT%TYPE,
      p_codcot   IN     RESULTAT.CODI_COTXE%TYPE,
      p_codcom   IN     RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN     RESULTAT.CODI_CARRERA%TYPE,
      p_datcar   IN     RESULTAT.DATA_CARRERA%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   );
END FIA_PKG;
/

CREATE OR REPLACE PACKAGE BODY GSIMONET.FIA_PKG
AS
   /******************************************************************************
     NOM:       FIA_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats que l'actor
                Administrador FIA pot executar des de l'aplicaci superior.
   ******************************************************************************/

   /* ***************************************************************************
       *  Descripci: Inserta/actualitza un resultat a la BD.
    *  Parmetros d'entrada
    *  p_codequ : Codi de l'equip
    *  p_codpil : Codi del pilot
    *  p_codcot : Cdi del cotxe
    *  p_codcom : Codi de la competici
    *  p_codcar : Codi de la carrera
    *  p_datcar : Data de la carrera
    *  p_codpos : Posici del pilot a la carrera.
    *  p_volrap : Volta rpida del pilot a la carrera
    *  Parmetros de sortida
    *  p_codres : Codi del resultat generat per la seqencia
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   PROCEDURE INSERT_UPDATE_RESULTAT (
      p_codequ   IN     RESULTAT.CODI_EQUIP%TYPE,
      p_codpil   IN     RESULTAT.CODI_PILOT%TYPE,
      p_codcot   IN     RESULTAT.CODI_COTXE%TYPE,
      p_codcom   IN     RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN     RESULTAT.CODI_CARRERA%TYPE,
      p_datcar   IN     RESULTAT.DATA_CARRERA%TYPE,
      p_codpos   IN     RESULTAT.POSICIO%TYPE,
      p_volrap   IN     RESULTAT.VOLTA_RAPIDA%TYPE,
      p_codres      OUT RESULTAT.CODI%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   )
   IS
      -- Variables pel control d'errors i excepcions
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      e_no_formacio EXCEPTION;
      e_no_cotxe EXCEPTION;
      e_no_carrera EXCEPTION;
      e_dates_distintes EXCEPTION;
      e_no_estadistica EXCEPTION;
      v_res      RESULTAT%ROWTYPE;
      v_est1     EST_VOLTES_RAPIDES%ROWTYPE;
      v_est2     EST_TEMP_MAXIMES%ROWTYPE;
      v_est3     EST_COM_PIL%ROWTYPE;

      CURSOR c_dates
      IS
         SELECT   COM.DATA_INICI, COM.DATA_FI
           FROM   COMPETICIO COM
          WHERE   COM.CODI = p_codcom;

      v_dates    c_dates%ROWTYPE;
   BEGIN
      p_codres := NULL;
      -- Parmetres de LOG
      v_nompro := 'INSERT_UPDATE_RESULTAT';
      v_log :=
            'p_codequ  ->'
         || p_codequ
         || 'p_codpil  ->'
         || p_codpil
         || 'p_codcot  ->'
         || p_codcot
         || 'p_codcom  ->'
         || p_codcom
         || 'p_codcar  ->'
         || p_codcar
         || 'p_datcar  ->'
         || p_datcar
         || 'p_codpos  ->'
         || p_codpos
         || 'p_volrap  ->'
         || p_volrap;

      -- Comprovar parmetre obligatoris

      IF    p_codequ IS NULL
         OR p_codpil IS NULL
         OR p_codcot IS NULL
         OR p_codcom IS NULL
         OR p_codcar IS NULL
         OR p_datcar IS NULL
      THEN
         RAISE e_null;
      END IF;

      -- Comprovar que la formaci existeix
      IF UTILITATS_PKG.EXISTEIX_FORMACIO (p_codpil,
                                          p_codequ,
                                          EXTRACT (YEAR FROM p_datcar)) = 'N'
      THEN
         RAISE e_no_formacio;
      END IF;

      -- Comprovar que el cotxe existeix
      IF UTILITATS_PKG.EXISTEIX_COTXE (p_codcot, p_codequ) = 'N'
      THEN
         RAISE e_no_cotxe;
      END IF;

      -- Comprovar que la competicio t associada la carrera
      IF UTILITATS_PKG.EXISTEIX_CARRERA (p_codcom, p_codcar) = 'N'
      THEN
         RAISE e_no_carrera;
      END IF;

      -- Comprovar que la data de la carrera = data volta rpida
      IF p_datcar <> (TO_DATE (TO_CHAR (p_volrap, 'DD/MM/YYYY')))
      THEN
         RAISE e_dates_distintes;
      END IF;

      -- Recuperar la row de RESULTATS
      v_res :=
         UTILITATS_PKG.OBTENIR_RESULTAT (p_codequ,
                                         p_codpil,
                                         p_codcot,
                                         p_codcom,
                                         p_codcar,
                                         p_datcar);

      -- Si no existeix la row --> INSERT
      IF v_res.CODI IS NULL
      THEN
         -- Generar codi de RESULTAT usant la seqncia
         SELECT   GSIMONET.RESULTAT_SQ.NEXTVAL INTO p_codres FROM DUAL;

         -- Insertar dades a RESULTAT
         INSERT INTO RESULTAT (CODI,
                               CODI_EQUIP,
                               CODI_PILOT,
                               CODI_COTXE,
                               CODI_COMPETICIO,
                               CODI_CARRERA,
                               DATA_CARRERA,
                               POSICIO,
                               VOLTA_RAPIDA)
           VALUES   (p_codres,
                     p_codequ,
                     p_codpil,
                     p_codcot,
                     p_codcom,
                     p_codcar,
                     p_datcar,
                     p_codpos,
                     p_volrap);
      ELSE
         -- Si existeix la row --> UPDATE
         UPDATE   RESULTAT
            SET   CODI_EQUIP = p_codequ,
                  CODI_PILOT = p_codpil,
                  CODI_COTXE = p_codcot,
                  CODI_COMPETICIO = p_codcom,
                  CODI_CARRERA = p_codcar,
                  DATA_CARRERA = p_datcar,
                  POSICIO = p_codpos,
                  VOLTA_RAPIDA = p_volrap
          WHERE   CODI = v_res.CODI;
      END IF;

      -- Insertar al mdul estadstic
      -- SELECT GSIMONET.EST_VOLTES_RAPIDES_SQ.NEXTVAL INTO v_codest FROM DUAL;

      -- Actualitza EST_VOLTES_RAPIDES
      v_est1.CODI_COMPETICIO := p_codcom;
      v_est1.CODI_CARRERA := p_codcar;

      ESTADISTICA_PKG.ACT_EST_VOLTES_RAPIDES (v_est1, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      -- Actualitza EST_COM_PIL
      OPEN c_dates;

      FETCH c_dates INTO   v_dates;

      CLOSE c_dates;

      v_est3.INICI_COMPETICIO := v_dates.DATA_INICI;
      v_est3.FINAL_COMPETICIO := v_dates.DATA_FI;
      v_est3.ANY_COMPETICIO := TO_NUMBER (EXTRACT (YEAR FROM p_datcar));
      v_est3.CODI_COMPETICIO := p_codcom;
      v_est3.CODI_EQUIP := p_codequ;
      v_est3.CODI_PILOT := p_codpil;
      v_est3.CODI_COTXE := p_codcot;

      ESTADISTICA_PKG.ACT_EST_COM_PIL (v_est3, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      COMMIT;
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_formacio
      THEN
         v_numerr := 'E007';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_cotxe
      THEN
         v_numerr := 'E008';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_carrera
      THEN
         v_numerr := 'E009';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_dates_distintes
      THEN
         v_numerr := 'E010';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_estadistica
      THEN
         v_numerr := 'E011';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END INSERT_UPDATE_RESULTAT;

   /* ***************************************************************************
          *  Descripci: Elimina un resultat de la BD.
    *  Parmetros d'entrada
    *  p_codequ : Codi de l'equip
    *  p_codpil : Codi del pilot
    *  p_codcot : Cdi del cotxe
    *  p_codcar : Codi de la carrera
    *  p_datcar : Data de la carrera
    *  Parmetros de sortida
    *  p_rsp:  'OK' en cas de execuci correcta
    *          'ERROR + TIPUS DE ERROR' en cas d'execuci errnia
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        01-04-2015   Creaci del procediment
    ****************************************************************************/
   PROCEDURE DELETE_RESULTAT (
      p_codequ   IN     RESULTAT.CODI_EQUIP%TYPE,
      p_codpil   IN     RESULTAT.CODI_PILOT%TYPE,
      p_codcot   IN     RESULTAT.CODI_COTXE%TYPE,
      p_codcom   IN     RESULTAT.CODI_COMPETICIO%TYPE,
      p_codcar   IN     RESULTAT.CODI_CARRERA%TYPE,
      p_datcar   IN     RESULTAT.DATA_CARRERA%TYPE,
      p_rsp         OUT LOGS.RSP%TYPE
   )
   IS
      -- Variables pel control d'errors i excepcions
      v_nompro   LOGS.NOM_PROCEDIMENT%TYPE;
      v_log      LOGS.LOG%TYPE;
      v_numerr   VARCHAR2 (30);
      v_msgerr   ERROR.DESCRIPCIO%TYPE;
      e_null EXCEPTION;
      e_no_resultat EXCEPTION;
      e_no_estadistica EXCEPTION;
      e_no_del_tel EXCEPTION;
      v_res      RESULTAT%ROWTYPE;
      v_est1     EST_VOLTES_RAPIDES%ROWTYPE;
      v_est2     EST_TEMP_MAXIMES%ROWTYPE;
      v_est3     EST_COM_PIL%ROWTYPE;
      v_est4     EST_NUM_COMP_FABRICANT%ROWTYPE;

      CURSOR c_dates
      IS
         SELECT   COM.DATA_INICI, COM.DATA_FI
           FROM   COMPETICIO COM
          WHERE   COM.CODI = p_codcom;

      v_dates    c_dates%ROWTYPE;

      CURSOR c_tels (p_codres IN TELEMETRIA.CODI_RESULTAT%TYPE)
      IS
         SELECT   TEL.CODI AS CODI_TELEMETRIA
           FROM   TELEMETRIA TEL
          WHERE   TEL.CODI_RESULTAT = p_codres;

      v_tels     c_tels%ROWTYPE;
   BEGIN
      -- Parmetres de LOG
      v_nompro := 'DELETE_RESULTAT';
      v_log :=
            'p_codequ  ->'
         || p_codequ
         || 'p_codpil  ->'
         || p_codpil
         || 'p_codcot  ->'
         || p_codcot
         || 'p_codcom  ->'
         || p_codcom
         || 'p_codcar  ->'
         || p_codcar;

      -- Comprovar parmetre obligatoris

      IF    p_codequ IS NULL
         OR p_codpil IS NULL
         OR p_codcot IS NULL
         OR p_codcom IS NULL
         OR p_codcar IS NULL
         OR p_datcar IS NULL
      THEN
         RAISE e_null;
      END IF;

      -- Recuperar la row de RESULTATS
      v_res :=
         UTILITATS_PKG.OBTENIR_RESULTAT (p_codequ,
                                         p_codpil,
                                         p_codcot,
                                         p_codcom,
                                         p_codcar,
                                         p_datcar);

      -- Si no existeix la row
      IF v_res.CODI IS NULL
      THEN
         RAISE e_no_resultat;
      ELSE
         -- Esborrar telemetries associades a aquest resultat
         FOR v_tels IN c_tels (v_res.CODI)
         LOOP
            TELEMETRIA_PKG.DELETE_TELEMETRIA (v_tels.CODI_TELEMETRIA, p_rsp);

            IF (p_rsp <> 'OK') AND (p_rsp <> 'E016')
            THEN
               RAISE e_no_del_tel;
            END IF;
         END LOOP;

         -- Esborrar resultat
         DELETE   RESULTAT RES
          WHERE   RES.CODI = v_res.CODI;
      END IF;

      -- Actualitza EST_VOLTES_RAPIDES
      v_est1.CODI_COMPETICIO := p_codcom;
      v_est1.CODI_CARRERA := p_codcar;

      ESTADISTICA_PKG.ACT_EST_VOLTES_RAPIDES (v_est1, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      -- Actualitza EST_COM_PIL
      OPEN c_dates;

      FETCH c_dates INTO   v_dates;

      CLOSE c_dates;

      v_est3.INICI_COMPETICIO := v_dates.DATA_INICI;
      v_est3.FINAL_COMPETICIO := v_dates.DATA_FI;
      v_est3.ANY_COMPETICIO := TO_NUMBER (EXTRACT (YEAR FROM p_datcar));
      v_est3.CODI_COMPETICIO := p_codcom;
      v_est3.CODI_EQUIP := p_codequ;
      v_est3.CODI_PILOT := p_codpil;
      v_est3.CODI_COTXE := p_codcot;

      ESTADISTICA_PKG.ACT_EST_COM_PIL (v_est3, p_rsp);

      IF p_RSP <> 'OK'
      THEN
         RAISE e_no_estadistica;
      END IF;

      p_rsp := 'OK';
      UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      COMMIT;
   EXCEPTION
      WHEN e_null
      THEN
         v_numerr := 'E003';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_resultat
      THEN
         v_numerr := 'E014';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_estadistica
      THEN
         v_numerr := 'E011';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN e_no_del_tel
      THEN
         v_numerr := 'E016';
         v_msgerr := UTILITATS_PKG.OBTENIR_MISSATGE_ERROR (v_numerr);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
      WHEN OTHERS
      THEN
         v_numerr := SQLCODE;
         v_msgerr := SUBSTR (SQLERRM, 1, 100);
         p_rsp := 'ERROR: ' || TO_CHAR (v_numerr) || ' ' || v_msgerr;
         UTILITATS_PKG.LOGS_INSERT (v_nompro, v_log, p_rsp);
   END DELETE_RESULTAT;
END FIA_PKG;
/

CREATE OR REPLACE PACKAGE GSIMONET.LLISTATS_PKG
AS
   /******************************************************************************
     NOM:       LLISTATS_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats perque tots
                els actor involucrats puguin realitzar les consultes amb cost 1
                del mduls estadstics.

                Cada funci correspon a un requisit funcional definit a la documen-
                taci i est indicat a seva capalera.
   ******************************************************************************/


   TYPE PER_COM_DEFECTUOS
   IS
      RECORD (
         CODI_COMPETICIO     EST_COM_PIL.CODI_COMPETICIO%TYPE,
         CODI_COTXE          EST_COM_PIL.CODI_COTXE%TYPE,
         CODI_PILOT          EST_COM_PIL.CODI_PILOT%TYPE,
         PER_COM_DEFECTUOS   EST_COM_PIL.PER_COM_DEFECTUOS%TYPE
      );

   TYPE PER_COM_DEFECTUOS_TAB IS TABLE OF PER_COM_DEFECTUOS;

   TYPE CLASSIFICACIO IS RECORD (
                            CODI_COMPETICIO   EST_COM_PIL.CODI_COMPETICIO%TYPE,
                            CODI_PILOT        EST_COM_PIL.CODI_PILOT%TYPE,
                            CLASSIFICACIO     EST_COM_PIL.CLASSIFICACIO%TYPE
                         );

   TYPE CLASSIFICACIO_TAB IS TABLE OF CLASSIFICACIO;

   TYPE NUM_CAR_ACABADES IS RECORD (
                               TOP                INTEGER,
                               CODI_PILOT         EST_COM_PIL.CODI_PILOT%TYPE,
                               NUM_CAR_ACABADES   EST_COM_PIL.NUM_CAR_ACABADES%TYPE
                            );

   TYPE NUM_CAR_ACABADES_TAB IS TABLE OF NUM_CAR_ACABADES;

   TYPE DINERS_PATROCINADOR IS RECORD (
                                  CODI_PATROCINADOR     EST_COM_PIL.CODI_PATROCINADOR%TYPE,
                                  DINERS_PATROCINADOR   EST_COM_PIL.DINERS_PATROCINADOR%TYPE
                               );

   TYPE DINERS_PATROCINADOR_TAB IS TABLE OF DINERS_PATROCINADOR;

   TYPE VOLUM_TELEMETRIA IS RECORD (
                               CODI_EQUIP         EST_COM_PIL.CODI_EQUIP%TYPE,
                               VOLUM_TELEMETRIA   EST_COM_PIL.VOLUM_TELEMETRIA%TYPE
                            );

   TYPE VOLUM_TELEMETRIA_TAB IS TABLE OF VOLUM_TELEMETRIA;

   TYPE NUM_CAR_GUANYADES IS RECORD (
                                TOP                 INTEGER,
                                CODI_PILOT          EST_COM_PIL.CODI_PILOT%TYPE,
                                NUM_CAR_GUANYADES   EST_COM_PIL.NUM_CAR_GUANYADES%TYPE
                             );

   TYPE NUM_CAR_GUANYADES_TAB IS TABLE OF NUM_CAR_GUANYADES;

   TYPE MILLOR_EVOLUCIO IS RECORD (
                              DIFERENCIA_PUNTS   INTEGER,
                              CODI_PILOT         EST_COM_PIL.CODI_PILOT%TYPE
                           );

   TYPE MILLOR_EVOLUCIO_PILOT_TAB IS TABLE OF MILLOR_EVOLUCIO;

   TYPE CARBURANT_GASTAT IS RECORD (
                               CODI_PILOT         EST_COM_PIL.CODI_PILOT%TYPE,
                               CODI_COTXE         EST_COM_PIL.CODI_COTXE%TYPE,
                               CARBURANT_GASTAT   EST_COM_PIL.CARBURANT_GASTAT%TYPE
                            );

   TYPE CARBURANT_GASTAT_TAB IS TABLE OF CARBURANT_GASTAT;

   TYPE VOLTA_RAPIDA IS RECORD (
                           VOLTA_RAPIDA   RESULTAT.VOLTA_RAPIDA%TYPE,
                           CODI_EQUIP     RESULTAT.CODI_EQUIP%TYPE,
                           CODI_PILOT     RESULTAT.CODI_PILOT%TYPE
                        );

   TYPE VOLTA_RAPIDA_TAB IS TABLE OF VOLTA_RAPIDA;

   TYPE TEMPERATURA_MAX
   IS
      RECORD (
         CODI_COMPETICIO   EST_TEMP_MAXIMES.CODI_COMPETICIO%TYPE,
         CODI_CARRERA      EST_TEMP_MAXIMES.CODI_COMPETICIO%TYPE,
         DATA_CARRERA      EST_TEMP_MAXIMES.DATA_CARRERA%TYPE,
         CODI_COTXE        EST_TEMP_MAXIMES.CODI_COTXE%TYPE,
         CODI_PILOT        EST_TEMP_MAXIMES.CODI_PILOT%TYPE,
         TEMPERATURA_MAX   EST_TEMP_MAXIMES.TEMPERATURA_MAX%TYPE
      );

   TYPE TEMPERATURA_MAX_TAB IS TABLE OF TEMPERATURA_MAX;

   TYPE NUM_COMPONENTS IS RECORD (
                             CODI_COMPETICIO   EST_NUM_COMP_FABRICANT.CODI_COMPETICIO%TYPE,
                             CODI_FABRICANT    EST_NUM_COMP_FABRICANT.CODI_FABRICANT%TYPE,
                             NUM_COMPONENTS    EST_NUM_COMP_FABRICANT.NUM_COMPONENTS%TYPE
                          );

   TYPE NUM_COMPONENTS_TAB IS TABLE OF NUM_COMPONENTS;

   /****************************************************************************
       * Requisit funcional 4.1 4.1.1. Obtenir  un llistat de percentatges dels seus
    * components defectuosos.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION LLISTAT_PER_COM_DEFECTUOSOS
      RETURN PER_COM_DEFECTUOS_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.2. Llistat dels resultats de cada pilot en cada
    * competici disputada.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION LLISTAT_RESULTATS_PILOTS
      RETURN CLASSIFICACIO_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.3. Llistat dels 10 pilots que han acabat ms carreres
    * entre totes les dades registrades.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP10_NUM_CAR_ACABADES
      RETURN NUM_CAR_ACABADES_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.4. Obtenir la classificaci de cada competici en curs
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION OBTENIR_CLASSIFICACIONS_CURS
      RETURN CLASSIFICACIO_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.5. Obtenir el Top-5 dels patrocinadors que aporten
    * ms diners als equips.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP5_PATROCINADORS_DINERS (
      p_anycom   IN EST_COM_PIL.ANY_COMPETICIO%TYPE
   )
      RETURN DINERS_PATROCINADOR_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.6.Obtenir lequip que ha recollit ms dades telemtriques.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION OBTENIR_EQUIP_VOLUM_TEL_MAX (
      p_anycom   IN EST_COM_PIL.ANY_COMPETICIO%TYPE
   )
      RETURN VOLUM_TELEMETRIA_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.7. Llistar el Top-10 de pilots pel que fa referncia a
    * curses guanyades entre totes les dades registrades.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP10_NUM_CAR_GUANYADES
      RETURN NUM_CAR_GUANYADES_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.8. Obtenir el pilot amb millor evoluci entre un
    * rang de dates.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION MILLOR_EVOLUCIO_PILOT (
      p_datini   IN EST_COM_PIL.INICI_COMPETICIO%TYPE,
      p_datfin   IN EST_COM_PIL.FINAL_COMPETICIO%TYPE
   )
      RETURN MILLOR_EVOLUCIO_PILOT_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.1.9. Obtenir el cotxe amb ms consum de carburant.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION OBTENIR_COTXE_CARB_GASTAT_MAX (
      p_codcom   IN EST_COM_PIL.CODI_COMPETICIO%TYPE,
      p_datcom   IN EST_COM_PIL.ANY_COMPETICIO%TYPE
   )
      RETURN CARBURANT_GASTAT_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.2.1. Obtenir  les 10 voltes ms rpides
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP10_VOLTES_RAPIDES (p_codcar RESULTAT.CODI_CARRERA%TYPE)
      RETURN VOLTA_RAPIDA_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.3.1.Obtenir la temperatura ms alta registrada al sistema
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION LLISTAT_TEMPERATURES_MAX_COM
      RETURN TEMPERATURA_MAX_TAB
      PIPELINED;

   /****************************************************************************
       * Requisit funcional 4.4.1. Llistar els 5 fabricants que aporten ms
    * components a la competici.
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP5_FABRICANTS_COMPONENTS
      RETURN NUM_COMPONENTS_TAB
      PIPELINED;
END LLISTATS_PKG;
/

CREATE OR REPLACE PACKAGE BODY GSIMONET.LLISTATS_PKG
AS
   /******************************************************************************
     NOM:       LLISTATS_PKG
     PROPSIT:  Package que cont tots els procediments emmagatzemats perque tots
                els actor involucrats puguin realitzar les consultes amb cost 1
                del mduls estadstics.

                Cada funci correspon a un requisit funcional definit a la documen-
                taci i est indicat a seva capalera.
   ******************************************************************************/

   /****************************************************************************
       * Requisit funcional 4.1 4.1.1. Obtenir  un llistat de percentatges dels seus
    * components defectuosos.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION LLISTAT_PER_COM_DEFECTUOSOS
      RETURN PER_COM_DEFECTUOS_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
         SELECT   EST.CODI_COMPETICIO,
                  EST.CODI_COTXE,
                  EST.CODI_PILOT,
                  EST.PER_COM_DEFECTUOS
           FROM   EST_COM_PIL EST;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END LLISTAT_PER_COM_DEFECTUOSOS;

   /****************************************************************************
             * Requisit funcional 4.1.2. Llistat dels resultats de cada pilot en cada
    * competici disputada.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION LLISTAT_RESULTATS_PILOTS
      RETURN CLASSIFICACIO_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
         SELECT   EST.CODI_COMPETICIO, EST.CODI_PILOT, EST.CLASSIFICACIO
           FROM   EST_COM_PIL EST;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END LLISTAT_RESULTATS_PILOTS;

   /****************************************************************************
             * Requisit funcional 4.1.3. Llistat dels 10 pilots que han acabat ms carreres
    * entre totes les dades registrades.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        02-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP10_NUM_CAR_ACABADES
      RETURN NUM_CAR_ACABADES_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   ROWNUM AS TOP, EST.CODI_PILOT, EST.NUM_CAR_ACABADES
             FROM   EST_COM_PIL EST
         ORDER BY   NUM_CAR_ACABADES DESC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END TOP10_NUM_CAR_ACABADES;

   /****************************************************************************
             * Requisit funcional 4.1.4. Obtenir la classificaci de cada competici en curs
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION OBTENIR_CLASSIFICACIONS_CURS
      RETURN CLASSIFICACIO_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
         SELECT   EST.CODI_COMPETICIO, EST.CODI_PILOT, EST.CLASSIFICACIO
           FROM   EST_COM_PIL EST
          WHERE   EST.ANY_COMPETICIO = EXTRACT (YEAR FROM SYSDATE);

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END OBTENIR_CLASSIFICACIONS_CURS;

   /****************************************************************************
             * Requisit funcional 4.1.5. Obtenir el Top-5 dels patrocinadors que aporten
    * ms diners als equips.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP5_PATROCINADORS_DINERS (
      p_anycom   IN EST_COM_PIL.ANY_COMPETICIO%TYPE
   )
      RETURN DINERS_PATROCINADOR_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   EST.CODI_PATROCINADOR,
                    SUM (EST.DINERS_PATROCINADOR) AS DINERS_PATROCINADOR_EQUIP
             FROM   EST_COM_PIL EST
            WHERE   EST.ANY_COMPETICIO = p_anycom
         GROUP BY   (EST.DINERS_PATROCINADOR, EST.CODI_PATROCINADOR)
         ORDER BY   DINERS_PATROCINADOR_EQUIP DESC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END TOP5_PATROCINADORS_DINERS;

   /****************************************************************************
             * Requisit funcional 4.1.6.Obtenir lequip que ha recollit ms dades telemtriques.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION OBTENIR_EQUIP_VOLUM_TEL_MAX (
      p_anycom   IN EST_COM_PIL.ANY_COMPETICIO%TYPE
   )
      RETURN VOLUM_TELEMETRIA_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   EST.CODI_EQUIP,
                    SUM (EST.VOLUM_TELEMETRIA) AS VOLUM_TELEMETRIA_EQUIP
             FROM   EST_COM_PIL EST
            WHERE   EST.ANY_COMPETICIO = p_anycom
         GROUP BY   (EST.VOLUM_TELEMETRIA, EST.CODI_EQUIP)
         ORDER BY   EST.VOLUM_TELEMETRIA DESC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END OBTENIR_EQUIP_VOLUM_TEL_MAX;

   /****************************************************************************
             * Requisit funcional 4.1.7. Llistar el Top-10 de pilots pel que fa referncia a
    * curses guanyades entre totes les dades registrades.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP10_NUM_CAR_GUANYADES
      RETURN NUM_CAR_GUANYADES_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   ROWNUM AS TOP, EST.CODI_PILOT, EST.NUM_CAR_GUANYADES
             FROM   EST_COM_PIL EST
         ORDER BY   EST.NUM_CAR_GUANYADES DESC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END TOP10_NUM_CAR_GUANYADES;

   /****************************************************************************
             * Requisit funcional 4.1.8. Obtenir el pilot amb millor evoluci entre un
    * rang de dates.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION MILLOR_EVOLUCIO_PILOT (
      p_datini   IN EST_COM_PIL.INICI_COMPETICIO%TYPE,
      p_datfin   IN EST_COM_PIL.FINAL_COMPETICIO%TYPE
   )
      RETURN MILLOR_EVOLUCIO_PILOT_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   MAX (EST3.CLASSIFICACIO_1 - EST3.CLASSIFICACIO_2)
                       AS MILLOR_EVOLUCIO,
                    EST3.CODI_PILOT
             FROM   (SELECT   EST1.CLASSIFICACIO AS CLASSIFICACIO_1,
                              EST2.CLASSIFICACIO AS CLASSIFICACIO_2,
                              EST1.CODI_PILOT
                       FROM   EST_COM_PIL EST1, EST_COM_PIL EST2
                      WHERE   p_datini BETWEEN EST1.INICI_COMPETICIO
                                           AND  EST1.FINAL_COMPETICIO
                              AND p_datfin BETWEEN EST2.INICI_COMPETICIO
                                               AND  EST2.FINAL_COMPETICIO
                              AND EST1.CODI_PILOT = EST2.CODI_PILOT) EST3
         GROUP BY   EST3.CLASSIFICACIO_1, EST3.CODI_PILOT
         ORDER BY   EST3.CLASSIFICACIO_1 ASC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END MILLOR_EVOLUCIO_PILOT;

   /****************************************************************************
             * Requisit funcional 4.1.9. Obtenir el cotxe amb ms consum de carburant.
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION OBTENIR_COTXE_CARB_GASTAT_MAX (
      p_codcom   IN EST_COM_PIL.CODI_COMPETICIO%TYPE,
      p_datcom   IN EST_COM_PIL.ANY_COMPETICIO%TYPE
   )
      RETURN CARBURANT_GASTAT_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   EST.CODI_PILOT, EST.CODI_COTXE, EST.CARBURANT_GASTAT
             FROM   EST_COM_PIL EST
            WHERE   EST.CODI_COMPETICIO = p_codcom
                    AND EST.ANY_COMPETICIO = p_datcom
         ORDER BY   CARBURANT_GASTAT DESC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END OBTENIR_COTXE_CARB_GASTAT_MAX;

   /****************************************************************************
             * Requisit funcional 4.2.1. Obtenir  les 10 voltes ms rpides
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP10_VOLTES_RAPIDES (p_codcar RESULTAT.CODI_CARRERA%TYPE)
      RETURN VOLTA_RAPIDA_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   RES.VOLTA_RAPIDA AS DATA_TEMPS,
                    RES.CODI_EQUIP,
                    RES.CODI_PILOT
             FROM   RESULTAT RES
            WHERE   RES.CODI_CARRERA = p_codcar
         ORDER BY   RES.VOLTA_RAPIDA ASC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END TOP10_VOLTES_RAPIDES;

   /****************************************************************************
             * Requisit funcional 4.3.1.Obtenir la temperatura ms alta registrada al sistema
    *
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION LLISTAT_TEMPERATURES_MAX_COM
      RETURN TEMPERATURA_MAX_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   EST.CODI_COMPETICIO,
                    EST.CODI_CARRERA,
                    EST.DATA_CARRERA,
                    EST.CODI_COTXE,
                    EST.CODI_PILOT,
                    EST.TEMPERATURA_MAX
             FROM   EST_TEMP_MAXIMES EST
            WHERE   EST.TEMPERATURA_MAX IS NOT NULL
         ORDER BY   EST.TEMPERATURA_MAX DESC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END LLISTAT_TEMPERATURES_MAX_COM;

   /****************************************************************************
             * Requisit funcional 4.4.1. Llistar els 5 fabricants que aporten ms
    * components a la competici.
    *  Autor            Data         Descripci
    *------------    ------------    ------------------------
    * GSIMONET        03-04-2015   Creaci de la funci
    ****************************************************************************/
   FUNCTION TOP5_FABRICANTS_COMPONENTS
      RETURN NUM_COMPONENTS_TAB
      PIPELINED
   IS
      CURSOR c_pip
      IS
           SELECT   EST.CODI_COMPETICIO, EST.CODI_FABRICANT, EST.NUM_COMPONENTS
             FROM   EST_NUM_COMP_FABRICANT EST
         ORDER BY   NUM_COMPONENTS DESC;

      v_pip   c_pip%ROWTYPE;
   BEGIN
      FOR v_pip IN c_pip
      LOOP
         --  Cargar linea en resultado
         PIPE ROW (v_pip);
      END LOOP;

      RETURN;
   END TOP5_FABRICANTS_COMPONENTS;
END LLISTATS_PKG;
/
